
# -*- coding: utf-8 -*-

################################################################################
#                                                                              #
#   This file is part of the Bibolamazi Project.                               #
#   Copyright (C) 2013 by Philippe Faist                                       #
#   philippe.faist@bluewin.ch                                                  #
#                                                                              #
#   Bibolamazi is free software: you can redistribute it and/or modify         #
#   it under the terms of the GNU General Public License as published by       #
#   the Free Software Foundation, either version 3 of the License, or          #
#   (at your option) any later version.                                        #
#                                                                              #
#   Bibolamazi is distributed in the hope that it will be useful,              #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of             #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              #
#   GNU General Public License for more details.                               #
#                                                                              #
#   You should have received a copy of the GNU General Public License          #
#   along with Bibolamazi.  If not, see <http://www.gnu.org/licenses/>.        #
#                                                                              #
################################################################################


import sys
import logging
import StringIO
import os.path
import re
import textwrap
import shlex
import logging

import bibolamazi.init
from bibolamazi.core import main as bibolamazimain
from bibolamazi.core import blogger
from bibolamazi.core.blogger import logger
from bibolamazi.core import bibolamazifile
from bibolamazi.core import butils
from bibolamazi.core.butils import BibolamaziError

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from .bibconfigsynthigh import BibolamaziConfigSyntaxHighlighter
from .favorites import FavoriteCmd, FavoritesModel, FavoritesOverBtns;
from . import filterinstanceeditor
from . import settingswidget

from .qtauto.ui_openbibfile import Ui_OpenBibFile

logger = logging.getLogger(__name__)



def bibolamazi_error_html(errortxt, bibolamaziFile, wrap_pre=True):

    def a_link(m):
        if bibolamaziFile is not None:
            return "<a href=\"action:/goto-config-line/%d\">%s</a>" %(
                bibolamaziFile.configLineNo(int(m.group('lineno'))),
                m.group()
                )
        return m.group();

    errortxt = str(Qt.escape(unicode(errortxt)))
    errortxt = re.sub(r'@:.*line\s+(?P<lineno>\d+)', a_link, errortxt)
    try:
        # if wrap_pre = (start_tag, end_tag)
        return wrap_pre[0] + errortxt + wrap_pre[1]
    except (TypeError,IndexError):
        pass
    if wrap_pre:
        # if wrap_pre = True
        return ("<pre>"+errortxt+"</pre>")
    return errortxt



class PreformattedHtml(object):
    def __init__(self, html):
        self.html = unicode(html)

    def getHtml(self):
        return self.html

    def __str__(self):
        return self.html.encode('utf-8')
    def __unicode__(self):
        return self.html

class LogToTextBrowserHandler(logging.Handler):
    def __init__(self, textEdit):
        logging.Handler.__init__(self)
        self.textEdit = textEdit

    def addtolog(self, txt, levelno=logging.INFO):
        #chfmt = QTextCharFormat()
        #if levelno == logging.ERROR or levelno == logging.CRITICAL:
        #    chfmt.setForeground(QColor(255,0,0));
        #    chfmt.setFontWeight(QFont.Bold);
        #elif levelno == logging.WARNING:
        #    chfmt.setForeground(QColor(204, 102, 0));
        #    chfmt.setFontWeight(QFont.Bold);
        #elif levelno == logging.INFO:
        #    chfmt.setForeground(QColor(0,0,0));
        #    chfmt.setFontWeight(QFont.Normal);
        #elif levelno == logging.DEBUG or levelno == logging.LONGDEBUG:
        #    chfmt.setForeground(QColor(127,127,127));
        #    chfmt.setFontWeight(QFont.Normal);
        #else:
        #    # unknown level
        #    chfmt.setForeground(QColor(127,127,127));
        #    chfmt.setFontWeight(QFont.Normal);

        try:
            html = txt.getHtml() # in case of a PreformattedHtml instance
        except AttributeError:
            html = unicode(Qt.escape(txt)) # in case of a simple plain text string

        sty = ''
        if levelno == logging.ERROR or levelno == logging.CRITICAL:
            sty = "color: #ff0000; font-weight: bold;"
        elif levelno == logging.WARNING:
            sty = "color: rgb(204,102,0); font-weight: bold;"
        elif levelno == logging.INFO:
            sty = "color: #000000; font-weight: normal;"
        elif levelno == logging.DEBUG or levelno == logging.LONGDEBUG:
            sty = "color: #7f7f7f; font-weight: normal;"
        else:
            # unknown level
            sty = "color: #7f7f7f; font-weight: normal;"

        sty += "white-space: pre;"

        cur = QTextCursor(self.textEdit.document())
        cur.movePosition(QTextCursor.End)
        #cur.setCharFormat(chfmt)
        #try:
        cur.insertHtml("<span style=\"%s\">%s\n</span>"%(sty, html))
        #except AttributeError: # txt is a normal string, not a PreformattedHtml instance
        #    cur.insertText(txt)
        self.textEdit.update()
        QApplication.instance().processEvents()

    def emit(self, record):
        self.addtolog(self.format(record), record.levelno)


class LogToTextBrowser:
    def __init__(self, textedit, bibolamaziFile, thelogger=logging.getLogger()):
        self.ch = LogToTextBrowserHandler(textedit);
        self.ch.setLevel(logging.NOTSET); # propagate all messages

        self.bibolamaziFile = bibolamaziFile

        # create formatter and add it to the handlers
        self.formatter = blogger.ConditionalFormatter('%(message)s',
                                                      DEBUG='-- %(message)s',
                                                      LONGDEBUG='  -- %(message)s',
                                                      WARNING='WARNING: %(message)s',
                                                      ERROR=self._fmt_error,
                                                      CRITICAL='CRITICAL: %(message)s');
        self.ch.setFormatter(self.formatter);

        self.logger = thelogger


    def _fmt_error(self, x):
        html = bibolamazi_error_html(
            '%(message)s' % x, bibolamaziFile=self.bibolamaziFile,
            wrap_pre=('<span style="white-space: pre">', '</span>'))
        html = "<br />".join(['ERROR: %s'%(line) for line in html.split('\n')])
        return PreformattedHtml(html=html)

    def addtolog(self, txt):
        self.ch.addtolog(txt)

    def __enter__(self):
        # add the handlers to the logger
        self.logger.addHandler(self.ch)
        self.ch.textEdit.clear()
        return self
        

    def __exit__(self, type, value, traceback):
        # clean-up
        self.logger.removeHandler(self.ch)





class ContextAttributeSetter:
    """Give a list of pairs of method and value to set.

    For example:

    >>> with ContextAttributeSetter( (object.isEnabled, object.setEnabled, False), ):
            ...

    will retreive the current state of if the object is enabled with `object.isEnabled()`, then
    will disable the object with `object.setEnabled(False)`. Upon exiting the with block, the
    state is restored to its original state with `object.setEnabled(..)`.

    """

    def __init__(self, *args):
        """Constructor. Does initializations. The \"enter\" statement is done with __enter__().

        Note: the argument are a list of 3-tuples `(get_method, set_method, set_to_value)`.
        """
        self.attribpairs = args
        self.initvals = None

    def __enter__(self):
        self.initvals = []
        for (getm, setm, v) in self.attribpairs:
            self.initvals.append(getm())
            setm(v)
            
        return self

    def __exit__(self, type, value, traceback):
        # clean-up
        for i in xrange(len(self.attribpairs)):
            (getm, setm, v) = self.attribpairs[i]
            setm(self.initvals[i])




class OpenBibFile(QWidget):

    requestHelpTopic = pyqtSignal('QString')

    fileClosed = pyqtSignal()
    

    def __init__(self):
        super(OpenBibFile, self).__init__()

        self.ui = Ui_OpenBibFile()
        self.ui.setupUi(self)

        self.resize(QSize(1200,800))
        self.ui.splitEditConfig.setSizes([100,1])

        self.ui.txtConfig.setWordWrapMode(QTextOption.WrapAnywhere)
        self.ui.txtLog.setWordWrapMode(QTextOption.WrapAnywhere)
        self.ui.txtBibEntries.setWordWrapMode(QTextOption.WrapAnywhere)

        font = settingswidget.get_typewriter_font(self.ui.txtConfig)
        self.ui.txtConfig.setFont(font)
        self.ui.txtLog.setFont(font)
        self.ui.txtBibEntries.setFont(font)

        self.syntHighlighter = BibolamaziConfigSyntaxHighlighter(self.ui.txtConfig)

        self._favorites_overbtn = FavoritesOverBtns(self.ui.treeFavorites)
        self._favorites_overbtn.insertCommand.connect(self._insert_new_cmd)

        self.ui.filterInstanceEditor.requestAddToFavorites.connect(self.add_favorite_cmd)
        self.ui.sourceListEditor.requestAddToFavorites.connect(self.add_favorite_cmd)

        self.bibolamaziFileName = None
        self.bibolamaziFile = None

        self.updateTimer = QTimer(self)
        self.updateTimer.setInterval(500)
        self.updateTimer.setSingleShot(True)
        QObject.connect(self.updateTimer, SIGNAL('timeout()'), self.reloadFile)
        
        self._flag_modified_externally = False

        self.fwatcher = QFileSystemWatcher(self)
        QObject.connect(self.fwatcher, SIGNAL('fileChanged(QString)'), self.fileModifiedExternally)
        #self.delayedUpdateFileContents)

        self.shortcuts = [
            QShortcut(QKeySequence('Ctrl+W'), self, self.close, self.close),
            QShortcut(QKeySequence('Ctrl+S'), self, self.saveToFile, self.saveToFile),
            QShortcut(QKeySequence(Qt.CTRL|Qt.Key_Return), self, self.runBibolamazi, self.runBibolamazi),
            ];

        self._ignore_change_for_edittools = False
        self._needs_update_txtbibentries = False
        self._set_modified(False)

        logger.longdebug("finished OpenBibFile constructor!")

    def setFavoriteCmdsList(self, favoriteCmdsList):
        self.favoriteCmdsList = favoriteCmdsList

        self._favorites_model = FavoritesModel(favcmds=self.favoriteCmdsList, parent=self);
        self.ui.treeFavorites.setModel(self._favorites_model);
        

    def setOpenFile(self, filename):
        if (self.bibolamaziFileName):
            self.fwatcher.removePath(self.bibolamaziFileName)
            
        self.bibolamaziFileName = filename
        self.bibolamaziFile = None

        self.reloadFile()

        if (self.bibolamaziFileName):
            self.fwatcher.addPath(self.bibolamaziFileName)


    def hasUnsavedModifications(self):
        return self._modified;

    def fileName(self):
        return self.bibolamaziFileName

    # use close() to close the widget and file
    def closeEvent(self, closeEvent):
        if (self._modified):
            ans = QMessageBox.question(self, 'Save Changes',
                                       "Save changes to file `%s'?" %(self.bibolamaziFileName),
                                       QMessageBox.Save|QMessageBox.Discard|QMessageBox.Cancel,
                                       QMessageBox.Save)
            if (ans == QMessageBox.Save):
                # save first (no need for updating), then proceed with close
                self.saveToFile(noupdate=True)
                
            if (ans == QMessageBox.Cancel):
                # ignore the event
                closeEvent.ignore()
                return
        
        if (self.bibolamaziFileName):
            self.fwatcher.removePath(self.bibolamaziFileName)
            self.bibolamaziFileName = None
        if (self.bibolamaziFile):
            self.bibolamaziFile = None

        self.fileClosed.emit()
        
        return super(OpenBibFile, self).closeEvent(closeEvent)


    @pyqtSlot()
    def saveToFile(self, noupdate=False):
        if (not self.bibolamaziFile):
            QMessageBox.critical(self, "No File!", "No file to save to!")
            return

        config_data = str(self.ui.txtConfig.toPlainText())

        # ignore changes caused by ourselves
        self.fwatcher.blockSignals(True)
        
        # change the config block.
        self.bibolamaziFile.setConfigData(config_data)
        self.bibolamaziFile.saveToFile()

        self._set_modified(False)

        # reload file.
        if (not noupdate):
            self.delayedUpdateFileContents()

    @pyqtSlot()
    def fileModifiedExternally(self):
        logger.debug("File modified externally!!")
        self._flag_modified_externally = True
        #self.delayedUpdateFileContents

    @pyqtSlot()
    def delayedUpdateFileContents(self):
        if (self.updateTimer.isActive()):
            self.updateTimer.stop()
        self.updateTimer.start()



    def _set_win_state(self, file_is_loaded, errormessagehtml=None, ifOkSetConfigTab=False):

        # enable/disable these GUI widgets/buttons
        
        self.ui.pageConfig.setEnabled(file_is_loaded)
        self.ui.pageBibEntries.setEnabled(file_is_loaded)
        self.ui.pageLog.setEnabled(file_is_loaded)

        self.ui.btnSave.setEnabled(file_is_loaded)
        self.ui.btnGo.setEnabled(file_is_loaded)
        

        if not file_is_loaded:
            # disabled state

            self.ui.tabs.setCurrentWidget(self.ui.pageInfo)
            self.ui.txtInfo.setText(errormessagehtml)

            with ContextAttributeSetter( (self.ui.txtConfig.signalsBlocked, self.ui.txtConfig.blockSignals, True) ):
                self.ui.txtConfig.setPlainText("")

            self.ui.lblFileName.setText("<no file loaded>")

            self.setWindowFilePath(QString())
            self.setWindowTitle("<no file loaded>")
            self.setWindowIcon(QIcon(':/pic/file.png'))

            self._modified = False

            return

        # normal, enabled state
        self.ui.lblFileName.setText(self.bibolamaziFileName)

        self.setWindowFilePath(self.bibolamaziFileName)
        self.setWindowTitle(os.path.basename(self.bibolamaziFileName))
        self.setWindowIcon(QIcon(':/pic/file.png'))

        if ifOkSetConfigTab:
            self.ui.tabs.setCurrentWidget(self.ui.pageConfig)
        self.ui.txtConfig.setFocus()


    @pyqtSlot()
    def reloadFile(self):
        logger.debug("reloading file")

        self._flag_modified_externally = False

        self._set_modified(False)

        # we may be sensitive again to external changes again
        if (self.fwatcher.signalsBlocked()):
            self.fwatcher.blockSignals(False)

        if (not self.bibolamaziFileName):
            with ContextAttributeSetter( (self.ui.btnGo.isEnabled, self.ui.btnGo.setEnabled, False) ):
                self.ui.txtInfo.setText("<h3 style=\"color: rgb(127,0,0)\">no file loaded.</h3>");
                self.ui.txtConfig.setPlainText("")
            return

        if (not self.bibolamaziFile):
            self.bibolamaziFile = bibolamazifile.BibolamaziFile()

        try:

            self.bibolamaziFile.load(self.bibolamaziFileName, to_state=bibolamazifile.BIBOLAMAZIFILE_READ)

        except butils.BibolamaziError as e:
            self.bibolamaziFile = None
            QMessageBox.critical(self, "Load Error", u"Error loading file: %s" %(unicode(e)))
            self._set_win_state(False,
                                "<h3 style=\"color: rgb(127,0,0)\">Error reading file.</h3>\n"
                                + bibolamazi_error_html(unicode(e), bibolamaziFile=self.bibolamaziFile))
            return

        self._set_win_state(True)

        with ContextAttributeSetter( (self.ui.txtConfig.signalsBlocked, self.ui.txtConfig.blockSignals, True) ):
            cursorpos = self.ui.txtConfig.textCursor().position()
            self.ui.txtConfig.setPlainText(self.bibolamaziFile.configData())
            cur = self.ui.txtConfig.textCursor()
            cur.setPosition(cursorpos)
            self.ui.txtConfig.setTextCursor(cur)

        self._needs_update_txtbibentries = True

        # now, try to further parse the config
        self._bibolamazifile_reparse()

        if self.bibolamaziFile.getLoadState() == bibolamazifile.BIBOLAMAZIFILE_PARSED:
            self._display_info()

        logger.debug("file contents updated!")

        

    def _display_info(self):

        def srcurl(s):
            if (re.match(r'^\w+:/', s)):
                # already URL
                return s;
            s = self.bibolamaziFile.resolveSourcePath(s)
            return 'file:///'+s;

        sources = []
        for srcline in self.bibolamaziFile.sourceLists():
            if (isinstance(srcline, list)):
                srclist = srcline
            else:
                srclist = [ srcline ]

            sources.append('''<div class="source">%(srcname)s: <ul>%(srclist)s</ul></div>''' % {
                'srcname': ('Source' if len(srclist) == 1 else 'Source List'), 
                'srclist': "".join(
                    ["<li><a href=\"%(sourceurl)s\">%(sourcepath)s</a></li>" % {
                        'sourcepath': Qt.escape(s),
                        'sourceurl': Qt.escape(srcurl(s)),
                        }
                     for s in srclist
                     ]
                    )
                });
                
        filters = []
        for fil in self.bibolamaziFile.filters():
            filters.append('''<div class="filter"><div class="filtertitle">Filter: <a href="helptopic:/filters/%(filtername)s">%(filtername)s</a></div><div class="filterdescription">%(filterdescription)s</div></div>''' % { 'filtername': fil.name(),
                     'filterdescription': fil.getHelpDescription(),
                    });

        thehtml = textwrap.dedent('''\
        <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">
        <html><head>
            <meta name="qrichtext" content="1" />
            <style type="text/css">
              body {
                white-space: normal;
                margin: 0px;
                padding: 0px;
              }
              .container {
                margin: 0px;
                padding-top: 20px;
              }
              h1 {
                font-weight: bold;
                font-size: 1.2em;
                color: rgb(0,127,127);
                margin: 0.5em 0px 0px;
                padding: 0px;
              }
              ul, li { margin: 0px; }
              p, li { white-space: normal }
              a { text-decoration: none; }
              .source { margin: 0.5em 0px 0px 0px; }
              .filter { margin: 0.3em 0px 0px 0px; }
              .filterdescription { font-style: italic; margin-left: 2.5em; }
            </style>
          </head><body><div class="container"><h1>Sources</h1>%(sourceshtml)s<h1>Filters</h1>%(filtershtml)s</div></body></html>''') % {
            'sourceshtml': "".join(sources),
            'filtershtml': "".join(filters)
            };

        self.ui.txtInfo.setHtml(thehtml)
        

    @pyqtSlot(int)
    def on_tabs_currentChanged(self, index):
        if (self.ui.tabs.widget(index) == self.ui.pageBibEntries):
            if (self._needs_update_txtbibentries):
                self.ui.txtBibEntries.setPlainText(self.bibolamaziFile.rawRest())


    @pyqtSlot()
    def on_btnGo_clicked(self):
        self.runBibolamazi()

    @pyqtSlot()
    def runBibolamazi(self):
        if (self._modified):
            yn = QMessageBox.question(self, "Save Changes?", "Save changes to your config?",
                                      QMessageBox.Save|QMessageBox.Cancel, QMessageBox.Save)
            if (yn == QMessageBox.Cancel):
                return

            # save our config
            self.saveToFile()
            
        with ContextAttributeSetter( (self.ui.btnGo.isEnabled, self.ui.btnGo.setEnabled, False) ):
            if (not self.bibolamaziFileName):
                QMessageBox.critical(self, "No open file", "No file selected!")
                return

            logs = None
            rootlogger = logging.getLogger()
            setloggerlevel = bibolamazimain.verbosity_logger_level(self.ui.cbxVerbosity.currentIndex())
            with ContextAttributeSetter( (lambda : rootlogger.level, rootlogger.setLevel,
                                          setloggerlevel),
                                         (self.ui.tabs.currentWidget, self.ui.tabs.setCurrentWidget,
                                          self.ui.pageLog) ):
                with LogToTextBrowser(textedit=self.ui.txtLog,
                                      bibolamaziFile=self.bibolamaziFile) as log2txtLog:
                    try:
                        # block notifications for file contents updates that we generate ourselves...
                        self.fwatcher.blockSignals(True)
                        bibolamazimain.run_bibolamazi(bibolamazifile=self.bibolamaziFileName)
                        log2txtLog.addtolog(" --> Finished successfully. <--")
                    except butils.BibolamaziError as e:
                        logger.error(unicode(e))
                        log2txtLog.addtolog(" --> Finished with errors. <--")
                        QMessageBox.warning(self, "Bibolamazi error", unicode(e))

        self.delayedUpdateFileContents()
                    


    @pyqtSlot()
    def on_btnSave_clicked(self):
        self.saveToFile()

    @pyqtSlot()
    def on_btnRefresh_clicked(self):
        if (self._modified):
            yn = QMessageBox.question(self, "Really revert?",
                                      self.tr("You are about to revert the file, but you "
                                              "have unsaved changes. Do you really want "
                                              "to abandon your changes and revert the "
                                              "file to disk?"),
                                      QMessageBox.Ok|QMessageBox.Cancel, QMessageBox.Cancel)
            if (yn != QMessageBox.Ok):
                return

        self.reloadFile()


    @pyqtSlot(QUrl)
    def on_txtInfo_anchorClicked(self, url):
        self._open_anchor(url)

    @pyqtSlot(QUrl)
    def on_txtLog_anchorClicked(self, url):
        self._open_anchor(url)

    def _open_anchor(self, url):
        if (url.scheme() == "helptopic"):
            self.requestHelpTopic.emit(url.path());
            return

        if (url.scheme() == "action"):
            m = re.match(r'/goto-config-line/(?P<lineno>\d+)', url.path())
            if m:
                self.ui.tabs.setCurrentWidget(self.ui.pageConfig)
                cur = QTextCursor(self.ui.txtConfig.document().findBlockByNumber(int(m.group('lineno'))-1))
                self.ui.txtConfig.setTextCursor(cur)
                return

            logger.warning("Unknown action: %s", str(url.toString()))
            return

        logger.debug("Opening URL %r", str(url.toString()))
        QDesktopServices.openUrl(url)
        

    @pyqtSlot(bool)
    def _set_modified(self, modif):
        self._modified = modif
        self.ui.btnSave.setEnabled(self._modified)

    @pyqtSlot()
    def on_txtConfig_textChanged(self):
        logger.debug("text changed! fwatcher signals blocked=%d", self.fwatcher.signalsBlocked())

        if self._flag_modified_externally:
            logger.debug("Modified externally!")
            
            revertbtn = None
            box = None
            if (not self._modified):
                # file was modified externally, but we have no local changes.
                box = QMessageBox(QMessageBox.Warning, "File Modified Externally",
                                  "The File was modified externally. Revert and reload it?")
                revertbtn = box.addButton("Revert", QMessageBox.AcceptRole)
                box.addButton(QMessageBox.Cancel)
                box.setDefaultButton(revertbtn)
            else:
                # file was modified externally, but we have LOCAL UNSAVED CHANGES
                box = QMessageBox(QMessageBox.Warning, "File Modified Externally",
                                  "The File was modified externally, but you have unsaved changes. "
                                  "Keep your changes and ignore the external modification, or "
                                  "revert it to version on disk?")
                revertbtn = box.addButton("Revert", QMessageBox.AcceptRole)
                box.addButton("Keep changes", QMessageBox.RejectRole)
                box.setDefaultButton(revertbtn)

            box.exec_()
            clickedbtn = box.clickedButton()
            if (clickedbtn == revertbtn):
                logger.debug("Reverting!!")
                # reload the file
                self.reloadFile()
                return

            self._flag_modified_externally = False

        if not self.bibolamaziFile:
            logger.debug("No file loaded.")
            return

        self._set_modified(True)
        logger.debug('modified!!')

        self.bibolamaziFile.setConfigData(str(self.ui.txtConfig.toPlainText()))
        self._bibolamazifile_reparse();

        self._do_update_edittools()


    def _bibolamazifile_reparse(self):
        try:
            self.bibolamaziFile.load(to_state=bibolamazifile.BIBOLAMAZIFILE_PARSED)
        except BibolamaziError as e:
            # see if we can parse the error
            self.ui.txtInfo.setHtml("<p style=\"color: rgb(127,0,0)\">Parse Error in file:</p>\n"+
                                    bibolamazi_error_html(unicode(e), bibolamaziFile=self.bibolamaziFile))
            return

    @pyqtSlot()
    def on_txtConfig_cursorPositionChanged(self):
        logger.debug("cursor position changed!")
        
        self._do_update_edittools()


    @pyqtSlot()
    def on_btnAddFilter_clicked(self):
        logger.debug('add filter: clicked')

        filter_list = filterinstanceeditor.get_filter_list()

        (filtname, ok) = QInputDialog.getItem(self, "Select Filter", "Please select which filter you wish to add",
                                              filter_list);
        if (not ok):
            # cancelled
            return

        # insert new filter command
        self._insert_new_cmd('filter: %s' %(filtname))

    @pyqtSlot()
    def on_btnAddSourceList_clicked(self):
        logger.debug('add source list: clicked')

        self._insert_new_cmd('src: ')

    def _insert_new_cmd(self, cmdtext):

        doc = self.ui.txtConfig.document()
        
        insertcur = None
        
        cmd = self._get_current_bibolamazi_cmd()
        if (cmd is None):
            insertcur = QTextCursor(doc.findBlockByNumber(self.ui.txtConfig.textCursor().block().blockNumber()))
        else:
            # insert _after_ current cmd (-> +1 for next line, +1 for counting from one and not from zero)
            insertcur = QTextCursor(doc.findBlockByNumber(self.bibolamaziFile.configLineNo(cmd.linenoend+1)+1))

        insertcur.insertText(str(cmdtext)+'\n')
        # select inserted text without the newline
        insertcur.movePosition(QTextCursor.Left)
        insertcur.movePosition(QTextCursor.StartOfBlock, QTextCursor.KeepAnchor)
        self.ui.txtConfig.setTextCursor(insertcur)


    def _get_current_bibolamazi_cmd(self):

        if (not self.bibolamaziFile):
            logger.warning("No bibolamazi file open")
            return

        cmds = self.bibolamaziFile.configCmds();
        if (cmds is None):
            # file is not yet parsed
            return None

        cur = self.ui.txtConfig.textCursor()
        block = cur.block()
        thisline = cur.block().blockNumber()+1

        thisline = self.bibolamaziFile.fileLineNo(thisline)

        logger.longdebug("searching for cmd... at file line=%d", thisline)
        for cmd in cmds:
            logger.longdebug("\t -- testing cmd %r", cmd)
            if cmd.lineno <= thisline and cmd.linenoend >= thisline:
                # got the current cmd
                logger.longdebug("\tGot cmd: %r", cmd)
                return cmd

        return None

    def _do_update_edittools(self):
        if self._ignore_change_for_edittools:
            return

        cmd = self._get_current_bibolamazi_cmd()
        if (cmd is None):
            self.ui.stackEditTools.setCurrentWidget(self.ui.toolspageBase)
            return
        
        if (cmd.cmd == 'src'):
            self.ui.sourceListEditor.setSourceList(shlex.split(cmd.text), noemit=True)
            self.ui.sourceListEditor.setRefDir(self.bibolamaziFile.fdir())
            self.ui.stackEditTools.setCurrentWidget(self.ui.toolspageSource)
            return

        if (cmd.cmd == "filter"):
            filtername = cmd.info['filtername']
            text = cmd.text
            if (text and text[-1] == '\n'):
                text = text[:-1]
            self.ui.filterInstanceEditor.setFilterInstanceDefinition(filtername, text,
                                                                     noemit=True)
            self.ui.stackEditTools.setCurrentWidget(self.ui.toolspageFilter)
            return

        logger.warning("Unknown command: %r", cmd)
        return


    def _replace_current_cmd(self, repltext, forcecheckcmd):
        logger.debug('_replace_current_cmd(%r,%r)', repltext, forcecheckcmd)
        
        cmd = self._get_current_bibolamazi_cmd()

        if (cmd is None or cmd.cmd != forcecheckcmd):
            logger.debug("Expected to currently be in cmd %s!!", forcecheckcmd)
            return

        logger.debug('About to change cmd %s on lines=%d--%d', cmd.cmd, cmd.lineno, cmd.linenoend)

        configlineno = self.bibolamaziFile.configLineNo(cmd.lineno)
        configlinenoend = self.bibolamaziFile.configLineNo(cmd.linenoend)

        self._ignore_change_for_edittools = True
        doc = self.ui.txtConfig.document()
        cursor = QTextCursor(doc.findBlockByNumber(configlineno-1))
        endblock = (configlinenoend+1)-1
        if (endblock >= doc.blockCount()):
            cursor.movePosition(QTextCursor.End, QTextCursor.KeepAnchor)
        else:
            cursorend = QTextCursor(doc.findBlockByNumber(endblock))
            cursor.setPosition(cursorend.position(), QTextCursor.KeepAnchor)
        cursor.insertText(repltext)
        tcursor = QTextCursor(doc.findBlockByNumber(configlineno-1))
        self.ui.txtConfig.setTextCursor(tcursor)
        self._ignore_change_for_edittools = False

        # now, reparse the config
        self.bibolamaziFile.setConfigData(str(self.ui.txtConfig.toPlainText()))
        self._bibolamazifile_reparse()
        

    @pyqtSlot(QStringList)
    def on_sourceListEditor_sourceListChanged(self, sourcelist):

        sourcelist = [str(x) for x in list(sourcelist)]

        cmdtext = "src: " + ("\n     ".join([butils.quotearg(x) for x in sourcelist])) + "\n"

        self._replace_current_cmd(cmdtext, 'src')


    @pyqtSlot()
    def on_filterInstanceEditor_filterInstanceDefinitionChanged(self):
        filtername = self.ui.filterInstanceEditor.filterName()
        optionstring = self.ui.filterInstanceEditor.optionString()

        logger.debug('on_filterInstanceEditor_filterInstanceDefinitionChanged() '
                     'filtername=%r, optionstring=%r', filtername, optionstring)

        cmdtext = "filter: " + filtername + ' ' + optionstring + "\n"

        self._replace_current_cmd(cmdtext, 'filter')
        

    @pyqtSlot(QString)
    def on_filterInstanceEditor_filterHelpRequested(self, topic):
        self.requestHelpTopic.emit(str(topic))


    @pyqtSlot()
    def add_favorite_cmd(self):
        cmd = self._get_current_bibolamazi_cmd()

        if (cmd is None):
            logger.warning("No command to add to favorites!")
            return

        logger.debug("Adding command %s on lines %d--%d to favorites: %r", cmd.cmd, cmd.lineno, cmd.linenoend, cmd)

        cmdtext = []
        doc = self.ui.txtConfig.document();
        for n in xrange(self.bibolamaziFile.configLineNo(cmd.lineno),
                        self.bibolamaziFile.configLineNo(cmd.linenoend)+1):
            cmdtext.append(str(doc.findBlockByNumber(n-1).text()))
        cmdtext = "\n".join(cmdtext)

        self.add_favorite_cmdtext(cmdtext)

    @pyqtSlot()
    def add_favorite_selection(self):
        # ### TODO : get selection (QTextCursor::selectedText()) ... and add favorite.
        #            Also add this option in the context menu.
        pass

    @pyqtSlot(QString)
    def add_favorite_cmdtext(self, cmdtext):
        cmdtext = str(cmdtext)
        
        self.favoriteCmdsList.addFavorite(FavoriteCmd(name=cmdtext[:30], cmd=cmdtext))

        QMessageBox.information(self, "Favorite Added", "Added this command to your favorites.")



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'favoritesoverbtns.ui'
#
# Created: Sun May 31 15:09:07 2015
#      by: PyQt4 UI code generator 4.10.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_FavoritesOverBtns(object):
    def setupUi(self, FavoritesOverBtns):
        FavoritesOverBtns.setObjectName(_fromUtf8("FavoritesOverBtns"))
        FavoritesOverBtns.resize(192, 32)
        FavoritesOverBtns.setWindowOpacity(0.8)
        self.horizontalLayout = QtGui.QHBoxLayout(FavoritesOverBtns)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setMargin(0)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        spacerItem = QtGui.QSpacerItem(5, 10, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.btnEndEdit = QtGui.QToolButton(FavoritesOverBtns)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/pic/ok.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnEndEdit.setIcon(icon)
        self.btnEndEdit.setIconSize(QtCore.QSize(10, 16))
        self.btnEndEdit.setObjectName(_fromUtf8("btnEndEdit"))
        self.horizontalLayout.addWidget(self.btnEndEdit)
        self.btnEdit = QtGui.QToolButton(FavoritesOverBtns)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnEdit.sizePolicy().hasHeightForWidth())
        self.btnEdit.setSizePolicy(sizePolicy)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(_fromUtf8(":/pic/lstbtnedit.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnEdit.setIcon(icon1)
        self.btnEdit.setIconSize(QtCore.QSize(10, 16))
        self.btnEdit.setObjectName(_fromUtf8("btnEdit"))
        self.horizontalLayout.addWidget(self.btnEdit)
        spacerItem1 = QtGui.QSpacerItem(20, 5, QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.btnDelete = QtGui.QToolButton(FavoritesOverBtns)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(_fromUtf8(":/pic/lstbtnremove.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnDelete.setIcon(icon2)
        self.btnDelete.setIconSize(QtCore.QSize(10, 16))
        self.btnDelete.setObjectName(_fromUtf8("btnDelete"))
        self.horizontalLayout.addWidget(self.btnDelete)
        self.btnInsert = QtGui.QPushButton(FavoritesOverBtns)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnInsert.sizePolicy().hasHeightForWidth())
        self.btnInsert.setSizePolicy(sizePolicy)
        self.btnInsert.setAutoFillBackground(True)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(_fromUtf8(":/pic/rightarrow.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnInsert.setIcon(icon3)
        self.btnInsert.setIconSize(QtCore.QSize(30, 16))
        self.btnInsert.setFlat(True)
        self.btnInsert.setObjectName(_fromUtf8("btnInsert"))
        self.horizontalLayout.addWidget(self.btnInsert)

        self.retranslateUi(FavoritesOverBtns)
        QtCore.QMetaObject.connectSlotsByName(FavoritesOverBtns)

    def retranslateUi(self, FavoritesOverBtns):
        self.btnEndEdit.setToolTip(_translate("FavoritesOverBtns", "Finish editing", None))
        self.btnEdit.setToolTip(_translate("FavoritesOverBtns", "Edit favorite commands", None))
        self.btnDelete.setToolTip(_translate("FavoritesOverBtns", "Delete this favorite command", None))
        self.btnInsert.setToolTip(_translate("FavoritesOverBtns", "Insert this favorite command at current position in config", None))
        self.btnInsert.setText(_translate("FavoritesOverBtns", "insert", None))

from . import bibolamazi_res_rc




# -*- coding: utf-8 -*-

################################################################################
#                                                                              #
#   This file is part of the Bibolamazi Project.                               #
#   Copyright (C) 2013 by Philippe Faist                                       #
#   philippe.faist@bluewin.ch                                                  #
#                                                                              #
#   Bibolamazi is free software: you can redistribute it and/or modify         #
#   it under the terms of the GNU General Public License as published by       #
#   the Free Software Foundation, either version 3 of the License, or          #
#   (at your option) any later version.                                        #
#                                                                              #
#   Bibolamazi is distributed in the hope that it will be useful,              #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of             #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              #
#   GNU General Public License for more details.                               #
#                                                                              #
#   You should have received a copy of the GNU General Public License          #
#   along with Bibolamazi.  If not, see <http://www.gnu.org/licenses/>.        #
#                                                                              #
################################################################################


import os
import os.path
import re
import logging

import bibolamazi.init
from bibolamazi.core.bibfilter import factory as filters_factory
from bibolamazi.core.bibfilter.factory import NoSuchFilter, NoSuchFilterPackage, FilterError
from bibolamazi.core import butils
from bibolamazi.core.bibfilter.argtypes import EnumArgType, CommaStrList

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from .qtauto.ui_filterinstanceeditor import Ui_FilterInstanceEditor

from . import overlistbuttonwidget

logger = logging.getLogger(__name__)


def get_filter_list():
    filter_pkg_list = filters_factory.detect_filter_package_listings()
    filter_list = []
    for (fpkg, flist) in filter_pkg_list.items():
        if fpkg == 'bibolamazi.filters':
            # built-in, ignore the filter package prefix.
            filter_list += flist
        else:
            filter_list += [ fpkg+':'+f for f in flist ]
    return filter_list




class RegisteredArgInputType:
    def __init__(self, typ, val):
        self.type_arg_input = typ.type_arg_input
        self.value = val

    def createWidget(self, parent, option):
        if (isinstance(self.type_arg_input, EnumArgType)):
            cbx = QComboBox(parent)
            for val in self.type_arg_input.listofvalues:
                cbx.addItem(val)

            return cbx

        logger.debug("Unknown type: type_arg_input=%r", self.type_arg_input)
        return None

    def setEditorData(self, editor):
        if (isinstance(self.type_arg_input, EnumArgType)):
            for i in xrange(editor.count()):
                if (str(editor.itemText(i)) == self.value):
                    editor.setCurrentIndex(i)
                    return
            return

        logger.debug("Unknown type: type_arg_input=%r", self.type_arg_input)
        return None

    def valueOf(self, editor):
        if (isinstance(self.type_arg_input, EnumArgType)):
            val = str(editor.itemText(editor.currentIndex()))
            logger.debug("GOT VALUE: %r", val)
            return val

        logger.debug("Unknown type: type_arg_input=%r", self.type_arg_input)
        return None
    


class DefaultFilterOptionsModel(QAbstractTableModel):
    def __init__(self, filtername=None, optionstring=None, parent=None):
        super(DefaultFilterOptionsModel, self).__init__(parent)
        self._filtername = None
        self._fopts = None
        self._optionstring = None
        
        self.setFilterName(filtername)


    optionStringChanged = pyqtSignal('QString')


    def optionstring(self):
        return self._optionstring

    @pyqtSlot(QString)
    def setFilterName(self, filtername, force=False, noemit=False, reset_optionstring=True):
        filtername = str(filtername)

        if (not force and self._filtername == filtername):
            return
        
        self._filtername = filtername

        self._fopts = None
        try:
            # remember: filter_uses_default_arg_parser() may also raise NoSuchFilter
            if (filtername and filters_factory.filter_uses_default_arg_parser(filtername)):
                self._fopts = filters_factory.DefaultFilterOptions(filtername)
        except (NoSuchFilter,NoSuchFilterPackage,FilterError) as e:
            logger.warning("No such filter, no such filter package or filtererror: %s", unicode(e))
            pass

        if (reset_optionstring):
            self.setOptionString('', force=True, noemit=noemit)
        else:
            self.setOptionString(self._optionstring, force=True, noemit=noemit)

        if (not noemit):
            # we shan't emit anything anyway for a filter name change.
            pass

    @pyqtSlot(QString)
    def setOptionString(self, optionstring, force=False, noemit=False):
        optionstring = str(optionstring);

        # don't reset source list if it's the same. In particular, don't emit the changed signal.
        if (not force and optionstring == self._optionstring):
            return

        if (self._fopts is None):
            # no managed options
            return

        # parse the options
        try:
            optspec = self._fopts.parse_optionstring_to_optspec(optionstring)
        except FilterError:
            return

        self._optionstring = optionstring

        pargs = optspec['_args']
        if pargs is None:
            pargs = []
        kwargs = optspec['kwargs']

        # NEW IN BIBOLAMAZI v3 : NO. pargs are ONLY given to the *args of the filter, and
        # never distributed to declared arguments as for a python function call. (That was
        # unnatural and confusing behavior.)
        #
        # # treat pargs as arguments to the function. These are usually declared arguments, simply
        # # provided without the key.
        # i = 0
        # argoptlist = self._fopts.filterDeclOptions()
        # while (len(pargs) and i < len(argoptlist)):
        #     # this parg corresponds to a kwarg.
        #     arg = argoptlist[i]
        #     if (arg.argname in kwargs):
        #         logger.warning("argument `%s' already given.", arg.argname)
        #         # don't pass this argument; stop argument parsing.
        #         break
        #     kwargs[arg.argname] = pargs.pop(0) # pop out first value into kwargs
        #     i = i + 1 # next declared argument in argoptlist

        self._pargs = pargs
        self._kwargs = kwargs

        self._emitLayoutChanged()


    @pyqtSlot('QString')
    def removeArgument(self, argname):
        argname = str(argname)
        
        logger.debug('remove argument: %r', argname)
        
        if (argname in self._kwargs):

            logger.debug('really removing argument!')
            
            del self._kwargs[argname]
            
            row = self.findArgByName(argname)
            idx = self.index(row, 2)
            self._update_optionstring()
            self.dataChanged.emit(idx,idx)
            self._emitOptionStringChanged()


    def rowCount(self, parent):
        if parent.isValid():
            return 0
        if (self._fopts is None):
            return 0
        return len(self._fopts.filterOptions())

    def columnCount(self, parent):
        if parent.isValid():
            return 0
        return 2
    

    #def _make_empty_type(self, arg):
    #    if (arg.argtypename is not None):
    #        fmodule = filters_factory.get_module(self._filtername, False)
    #        typ = butils.resolve_type(arg.argtypename, fmodule)
    #        return typ()
    #    return str('')
        

    def data(self, index, role=Qt.DisplayRole):
        if (self._fopts is None):
            return QVariant()
        
        filteroptions = self._fopts.filterOptions()

        col = index.column()
        row = index.row()

        if (row < 0 or row >= len(filteroptions)):
            return QVariant()

        # the argument specification of the current row (_ArgDoc namedtuple instance)
        arg = filteroptions[row]

        if (col == 0):
            # argument name
            if (role == Qt.DisplayRole):
                return QVariant(QString(self._fopts.getSOptNameFromArg(filteroptions[row].argname)))

            # tool-tip documentation
            if (role == Qt.ToolTipRole):
                return QVariant(QString(arg.doc))

            return QVariant()

        # the value of the argument of the current row.
        val = self._kwargs.get(arg.argname)
        
        if (col == 1):
            # argument value

            if (role == overlistbuttonwidget.ROLE_OVERBUTTON):
                if (val is not None):
                    return QVariant(overlistbuttonwidget.OVERBUTTON_REMOVE)
                return QVariant(overlistbuttonwidget.OVERBUTTON_ADD)

            if (role == overlistbuttonwidget.ROLE_ARGNAME):
                return QVariant(QString(arg.argname))
            
            if (role == Qt.DisplayRole):
                if (val is None):
                    return QVariant()
                return QVariant(QString(str(val)))

            # request editing value of argument
            if (role == Qt.EditRole):
                if (arg.argtypename is not None):
                    fmodule = filters_factory.get_module(self._filtername, False)
                    typ = butils.resolve_type(arg.argtypename, fmodule)
                else:
                    typ = str
                if (hasattr(typ, 'type_arg_input')):
                    editval = RegisteredArgInputType(typ, val)
                elif (issubclass(typ, basestring) and val is None):
                    editval = typ('')
                elif (issubclass(typ, CommaStrList)):
                    editval = unicode(val)
                else:
                    editval = typ(val)
                return QVariant(editval)
            return QVariant()

        return QVariant()


    def headerData(self, section, orientation, role=Qt.DisplayRole):
        if (orientation == Qt.Vertical):
            return QVariant()

        if (section == 0):
            if (role == Qt.DisplayRole):
                return QVariant(QString(u"Filter Option"))
            return QVariant()

        if (section == 1):
            if (role == Qt.DisplayRole):
                return QVariant(QString(u"Value"))
            return QVariant()

        return QVariant()


    def flags(self, index):
        col = index.column()
        row = index.row()

        if (self._fopts is None):
            return Qt.NoItemFlags

        if (col == 0):
            return Qt.ItemIsSelectable | Qt.ItemIsEnabled
        if (col == 1):
            return Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsEditable

        logger.debug("DefaultFilterOptionsModel.flags(): BAD COLUMN: %d", col)
        return 0
    

    def setData(self, index, value, role=Qt.EditRole):
        
        col = index.column()
        row = index.row()

        if (col != 1):
            return False

        if (self._fopts is None):
            return False

        if (role != Qt.EditRole):
            return False

        filteroptions = self._fopts.filterOptions()

        if (row < 0 or row >= len(filteroptions)):
            return False

        arg = filteroptions[row]

        value = value.toPyObject()
        
        if (isinstance(value, QString)):
            value = unicode(value); # make sure we're dealing with Python strings and not Qt strings

        logger.debug("Got value: %r", value)

        # validate type
        typ = None
        if (arg.argtypename is not None):
            typ = butils.resolve_type(arg.argtypename, filters_factory.get_module(self._filtername, False))
        if (typ == None):
            typ = str
            
        value = typ(value)

        logger.debug("Got final value: %r ; typ=%r", value, typ)

        self._kwargs[arg.argname] = value

        self._update_optionstring()

        self.dataChanged.emit(index, index)

        self._emitOptionStringChanged()

        logger.debug("_kwargs is %r", self._kwargs)
        return True

    def findArgByName(self, argname):
        filteroptions = self._fopts.filterOptions()

        for row in xrange(len(filteroptions)):
            if (filteroptions[row].argname == argname):
                return row

        return None
    

    def _update_optionstring(self):
        slist = []
        for arg in self._pargs:
            slist.append(butils.quotearg(arg))
            
        # iterate fopts arguments to preserve that ordering
        done_args = []
        for arg in self._fopts.filterOptions():
            v = self._kwargs.get(arg.argname,None)
            if (v is not None):
                soptarg = self._fopts.getSOptNameFromArg(arg.argname)
                if (arg.argtypename == 'bool'):
                    slist.append('-d'+soptarg+('' if v else '=False'))
                else:
                    slist.append('-s'+soptarg+'='+butils.quotearg(str(v)))
                    
            done_args.append(arg.argname)

        # and add any extra args
        for (k,v) in self._kwargs.iteritems():
            if (k in done_args):
                continue
            slist.append('-s' + k + '=' + butils.quotearg(str(v)))

        self._optionstring = " ".join(slist)

        logger.debug("option string is now %r", self._optionstring)
            

    def _emitOptionStringChanged(self):
        self.optionStringChanged.emit(self._optionstring)

    def _emitLayoutChanged(self):
        self.layoutChanged.emit()



class DefaultFilterOptionsDelegate(QStyledItemDelegate):
    def __init__(self, parentView=None):
        super(DefaultFilterOptionsDelegate, self).__init__(parentView)
        self._view = parentView


    def createEditor(self, parent, option, index):
        if (index.column() != 1):
            return super(DefaultFilterOptionsDelegate, self).createEditor(parent, option, index)
        
        data = index.data(Qt.EditRole)
        if (isinstance(data.toPyObject(), RegisteredArgInputType)):
            # do our own processing with some custom widgets.
            rr = data.toPyObject()
            w = rr.createWidget(parent, option)
            w.setProperty('_RegisteredArgInputType', data)
            return w
        
        return super(DefaultFilterOptionsDelegate, self).createEditor(parent, option, index)
    
    def updateEditorGeometry(self, editor, option, index):
        editor.setGeometry(option.rect)
    
    def setEditorData(self, editor, index):
        rr = editor.property('_RegisteredArgInputType')
        if (not rr.isValid()):
            return super(DefaultFilterOptionsDelegate, self).setEditorData(editor, index)
        
        rr = rr.toPyObject()
        rr.setEditorData(editor) # the value is already contained in the rr object

    def setModelData(self, editor, model, index):
        rr = editor.property('_RegisteredArgInputType')
        if (not rr.isValid()):
            return super(DefaultFilterOptionsDelegate, self).setModelData(editor, model, index)
        
        rr = rr.toPyObject()
        model.setData(index, QVariant(rr.valueOf(editor)))





class FilterInstanceEditor(QWidget):
    def __init__(self, parent):
        super(FilterInstanceEditor, self).__init__(parent)

        self._is_updating = True

        self.ui = Ui_FilterInstanceEditor()
        self.ui.setupUi(self)
        
        for filtername in get_filter_list():
            self.ui.cbxFilter.addItem(filtername)

        self.ui.btnAddFavorite.clicked.connect(self.requestAddToFavorites)

        self.filterNameChanged.connect(self.filterInstanceDefinitionChanged)
        self.filterOptionsChanged.connect(self.filterInstanceDefinitionChanged)

        self._filteroptionsmodel = DefaultFilterOptionsModel(filtername=None, parent=self)

        self._filteroptionsdelegate = DefaultFilterOptionsDelegate(parentView=self.ui.lstOptions)
        
        self.ui.lstOptions.setModel(self._filteroptionsmodel)
        self.ui.lstOptions.setItemDelegate(self._filteroptionsdelegate)

        self._filteroptionsmodel.optionStringChanged.connect(self.filterOptionsChanged)
        ##self._filteroptionsmodel.dataChanged.connect(self._filteroptionsdelegate.update_buttons)

        self._filterargbtn = overlistbuttonwidget.OverListButtonWidget(self.ui.lstOptions)
        self._filterargbtn.removeClicked.connect(self._filteroptionsmodel.removeArgument)
        self._filterargbtn.addIndexClicked.connect(self.ui.lstOptions.edit)
        self._filterargbtn.editIndexClicked.connect(self.ui.lstOptions.edit)

        self._filteroptionsmodel.optionStringChanged.connect(self._filterargbtn.updateDisplay)

        self._is_updating = False
        

    filterInstanceDefinitionChanged = pyqtSignal()
    filterNameChanged = pyqtSignal('QString')
    filterOptionsChanged = pyqtSignal('QString')

    filterHelpRequested = pyqtSignal('QString')

    requestAddToFavorites = pyqtSignal()
    

    def filterName(self):
        return str(self.ui.cbxFilter.currentText())

    def optionString(self):
        return self._filteroptionsmodel.optionstring()


    @pyqtSlot(QString, bool)
    @pyqtSlot(QString)
    def setFilterName(self, filtername, noemit=False, force=False, reset_optionstring=True):
        logger.debug("setFilterName(%r)", filtername)
        if (not force and self.ui.cbxFilter.currentText() == filtername):
            return
        
        self._is_updating = True
        self.ui.cbxFilter.setEditText(filtername)
        self._is_updating = False

        self._filteroptionsmodel.setFilterName(filtername, noemit=True,
                                               reset_optionstring=reset_optionstring)

        self.ui.lstOptions.resizeColumnToContents(0)
        
        if (not noemit):
            self.emitFilterNameChanged()


    @pyqtSlot(QString)
    def setOptionString(self, optionstring, noemit=False, force=False):
        self._filteroptionsmodel.setOptionString(optionstring, force=force, noemit=noemit)
        self.ui.lstOptions.resizeColumnToContents(0)
        self.ui.lstOptions.setColumnWidth(2, 20)
        ##self._filteroptionsdelegate.update_buttons()

    def setFilterInstanceDefinition(self, filtername, optionstring, noemit=False):
        self.setFilterName(filtername, noemit=noemit, force=True, reset_optionstring=True)
        self.setOptionString(optionstring, noemit=noemit)

    @pyqtSlot()
    def emitFilterNameChanged(self):
        if (self._is_updating):
            return
        logger.debug("emitting filterNameChanged! filterName=%s", self.filterName())
        self.filterNameChanged.emit(QString(self.filterName()))

    @pyqtSlot(QString)
    def on_cbxFilter_editTextChanged(self, s):
        if (self._is_updating):
            return
        self.setFilterName(s, force=True, reset_optionstring=False)


    @pyqtSlot()
    def on_btnFilterHelp_clicked(self):
        self.filterHelpRequested.emit('filters/%s' %(QString(self.filterName())))

        




# -*- coding: utf-8 -*-

################################################################################
#                                                                              #
#   This file is part of the Bibolamazi Project.                               #
#   Copyright (C) 2014 by Philippe Faist                                       #
#   philippe.faist@bluewin.ch                                                  #
#                                                                              #
#   Bibolamazi is free software: you can redistribute it and/or modify         #
#   it under the terms of the GNU General Public License as published by       #
#   the Free Software Foundation, either version 3 of the License, or          #
#   (at your option) any later version.                                        #
#                                                                              #
#   Bibolamazi is distributed in the hope that it will be useful,              #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of             #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              #
#   GNU General Public License for more details.                               #
#                                                                              #
#   You should have received a copy of the GNU General Public License          #
#   along with Bibolamazi.  If not, see <http://www.gnu.org/licenses/>.        #
#                                                                              #
################################################################################

from collections import namedtuple
import logging

from PyQt4.QtCore import *
from PyQt4.QtGui import *

import bibolamazi.init
from .overlistbuttonwidget import OverListButtonWidgetBase
from .qtauto.ui_favoritesoverbtns import Ui_FavoritesOverBtns

logger = logging.getLogger(__name__)


class FavoriteCmd:
    def __init__(self, name, cmd):
        self.name = name
        self.cmd = cmd


default_cmds = [
    FavoriteCmd(name="fixes (standard; for LaTeX)",
                cmd=("filter: fixes -dFixSwedishA -dEncodeUtf8ToLatex -dRemoveTypeFromPhd "
                     "-sRemoveFullBraces=title -sProtectNames=Einstein,Maxwell,Landauer,"
                     "Shannon,Neumann,Szilard,Bell,I,II,III,IV,XIV -dRemoveFileField")),
    FavoriteCmd(name="arxiv (for notes & verbose)",
                cmd="filter: arxiv -sMode=eprint -sUnpublishedMode=eprint -dThesesCountAsPublished -dWarnJournalRef"),
    FavoriteCmd(name="arxiv (for articles & finalizing)",
                cmd="filter: arxiv -sMode=strip -sUnpublishedMode=unpublished-note"),
    FavoriteCmd(name="url (standard cleanup)",
                cmd="filter: url -dStripAllIfDoiOrArxiv -dStripDoiUrl -dStripArxivUrl -dKeepFirstUrlOnly"),
    FavoriteCmd(name="orderentries (alphabetically by cite key)",
                cmd="filter: orderentries -sOrder=alphabetical"),
    ];

class FavoriteCmdsList(QObject):
    def __init__(self, parent):
        super(FavoriteCmdsList,self).__init__(parent)
        self.favlist = [];

    favChanged = pyqtSignal()

    def loadDefaults(self):
        self.favlist[:] = default_cmds

    def loadFromSettings(self, settings):
        settings.beginGroup("Favorites")

        ok = settings.contains("has_favorites") and settings.value("has_favorites").toBool()
        if not ok:
            logger.debug("Loading defaults for favorites")
            self.loadDefaults()
            return

        self.favlist[:] = []
        
        siz = settings.beginReadArray("favorite_cmds")
        for i in xrange(siz):
            settings.setArrayIndex(i)
            self.favlist.append(FavoriteCmd(name=str(settings.value("name").toString()),
                                            cmd=str(settings.value("cmd").toString())))
        settings.endArray()

        settings.endGroup()

        self.favChanged.emit()
        

    def saveToSettings(self, settings):
        settings.beginGroup("Favorites")

        settings.setValue("has_favorites", QVariant(True))

        settings.beginWriteArray("favorite_cmds", len(self.favlist))
        for i in xrange(len(self.favlist)):
            fav = self.favlist[i]
            settings.setArrayIndex(i)
            settings.setValue("name", QVariant(QString(fav.name)))
            settings.setValue("cmd", QVariant(QString(fav.cmd)))
        settings.endArray()

        settings.endGroup()

    def addFavorite(self, favcmd):
        self.favlist += [favcmd]
        self.favChanged.emit()

    def moveFavorite(self, oldind, newind):
        if oldind == newind:
            return

        if oldind < newind:
            self.favlist[oldind:newind] = self.favlist[oldind+1:newind] + [self.favlist[oldind]]
        else: # oldind > newind
            self.favlist[newind:oldind+1] = [self.favlist[oldind]] + self.favlist[newind:oldind]

        self.favChanged.emit()




ROLE_CMD            = Qt.UserRole + 2014
ROLE_FAV_EDITABLE   = ROLE_CMD + 1


class FavoritesModel(QAbstractTableModel):
    def __init__(self, favcmds, parent):
        super(FavoritesModel, self).__init__(parent=parent)
        self._favcmds = favcmds;
        self._favcmds.favChanged.connect(self.modelReset)
        self._edit_mode = False

    def setEditMode(self, editmodeon):
        self._edit_mode = editmodeon;
        self.modelReset.emit()

    def rowCount(self, index=QModelIndex()):
        if index.isValid():
            return 0;
        return len(self._favcmds.favlist);

    def columnCount(self, index=QModelIndex()):
        if index.isValid():
            return 0;
        return 1;

    def data(self, index, role=Qt.DisplayRole):

        if role == Qt.BackgroundRole:
            if self._edit_mode:
                return QVariant(QBrush(QColor(255,230,230)))
            return QVariant()

        if not index.isValid():
            return QVariant()
        
        col = index.column()
        row = index.row()

        if (row < 0 or row >= len(self._favcmds.favlist)):
            return QVariant()

        if (role == ROLE_FAV_EDITABLE):
            return QVariant(self._edit_mode)

        if (role == ROLE_CMD):
            return QVariant(QString(self._favcmds.favlist[row].cmd))

        # argument name
        if (role == Qt.DisplayRole):
            return QVariant(QString(self._favcmds.favlist[row].name))

        # editable name
        if (role == Qt.EditRole):
            return QVariant(QString(self._favcmds.favlist[row].name))
        
        # tool-tip documentation
        if (role == Qt.ToolTipRole):
            return QVariant(QString(self._favcmds.favlist[row].cmd))

        return QVariant()

    def setData(self, index, value, role=Qt.EditRole):
        if not index.isValid():
            return False
        
        col = index.column()
        row = index.row()

        if (row < 0 or row >= len(self._favcmds.favlist)):
            return False

        self._favcmds.favlist[row].name = str(value.toString())
        self.modelReset.emit()

        

    def removeFavorite(self, row):
        if (isinstance(row, QModelIndex)):
            row = row.row()
        if row < 0 or row >= len(self._favcmds.favlist):
            return
        del self._favcmds.favlist[row]
        self.modelReset.emit()

    def headerData(self, section, orientation, role=Qt.DisplayRole):
        if (orientation == Qt.Vertical):
            return QVariant()

        if (section == 0):
            if (role == Qt.DisplayRole):
                return QVariant(QString(u""))
            return QVariant()

        if (section == 1):
            if (role == Qt.DisplayRole):
                return QVariant(QString(u""))
            return QVariant()

        return QVariant()


    def flags(self, index):
        if self._edit_mode:
            if not index.isValid():
                return Qt.ItemIsDropEnabled
            return (Qt.ItemIsSelectable | Qt.ItemIsEditable | Qt.ItemIsEnabled | Qt.ItemIsDragEnabled)

        return Qt.ItemIsEnabled


    def supportedDropActions(self):
        return Qt.MoveAction

    def mimeTypes(self):
        return QStringList() << "application/x-bibolamazi-internalmove-favorites";

    def mimeData(self, indexes):
        logger.longdebug("indexes=%r", indexes)
        if len(indexes) != 1:
            return None

        mimeData = QMimeData()
        mimeData.setData('application/x-bibolamazi-internalmove-favorites',
                         QByteArray(str('%d' %(indexes[0].row()))))
        logger.longdebug("mimeData=%r", mimeData)
        return mimeData

    def dropMimeData(self, data, action, row, column, parent):
        logger.longdebug(
            "Drop! data=%r (formats=%r), action=%r, row=%r, column=%r, parent=%r (%d,%d)",
            data, list(data.formats()), action, row, column, parent, parent.row(), parent.column()
        )

        if not data.hasFormat('application/x-bibolamazi-internalmove-favorites'):
            return False

        if row < 0 or parent.isValid():
            return False

        oldrow = int(data.data("application/x-bibolamazi-internalmove-favorites"))
        logger.longdebug("oldrow: %r", oldrow)

        self._favcmds.moveFavorite(oldrow, row);
        return True



class FavoritesOverBtns(OverListButtonWidgetBase):
    def __init__(self, itemview):
        super(FavoritesOverBtns, self).__init__(itemview=itemview)

        self.ui = Ui_FavoritesOverBtns()
        self.ui.setupUi(self)
        

    insertCommand = pyqtSignal('QString')

    def show_status(self, idx):
        if not idx.isValid():
            return False

        if idx.data(ROLE_FAV_EDITABLE).toBool():
            # edit mode
            self.ui.btnInsert.hide()
            self.ui.btnEdit.hide()
            self.ui.btnEndEdit.show()
            self.ui.btnDelete.show()
            return True

        # else: normal, not editable
        self.ui.btnInsert.show()
        self.ui.btnEdit.show()
        self.ui.btnEndEdit.hide()
        self.ui.btnDelete.hide()
        return True

    @pyqtSlot()
    def on_btnInsert_clicked(self):
        curidx = self.curIndex()
        if not curidx.isValid():
            return

        self.insertCommand.emit( curidx.data(ROLE_CMD).toString() )

    @pyqtSlot()
    def on_btnEdit_clicked(self):
        self.itemView().model().setEditMode(True)
        self.itemView().setCurrentIndex(self.curIndex())
        self.updateDisplay()

    @pyqtSlot()
    def on_btnEndEdit_clicked(self):
        self.itemView().model().setEditMode(False)
        self.updateDisplay()


    @pyqtSlot()
    def on_btnDelete_clicked(self):
        curidx = self.curIndex()
        if QMessageBox.question(self, "Confirm deletion",
                                "Are you sure you want to delete this favorite command?",
                                QMessageBox.Yes|QMessageBox.No,
                                QMessageBox.No) != QMessageBox.Yes:
            return

        curidx.model().removeFavorite(curidx)
        self.updateDisplay()


    def get_widget_rect(self, rect):
        return rect



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'settingswidget.ui'
#
# Created: Sun May 31 15:09:07 2015
#      by: PyQt4 UI code generator 4.10.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_SettingsWidget(object):
    def setupUi(self, SettingsWidget):
        SettingsWidget.setObjectName(_fromUtf8("SettingsWidget"))
        SettingsWidget.resize(501, 285)
        self.verticalLayout_2 = QtGui.QVBoxLayout(SettingsWidget)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.gbxFilterPackages = QtGui.QTabWidget(SettingsWidget)
        self.gbxFilterPackages.setObjectName(_fromUtf8("gbxFilterPackages"))
        self.tabFilterPackages = QtGui.QWidget()
        self.tabFilterPackages.setObjectName(_fromUtf8("tabFilterPackages"))
        self.gridLayout = QtGui.QGridLayout(self.tabFilterPackages)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.btnFilterPackageAdd = QtGui.QPushButton(self.tabFilterPackages)
        self.btnFilterPackageAdd.setObjectName(_fromUtf8("btnFilterPackageAdd"))
        self.horizontalLayout.addWidget(self.btnFilterPackageAdd)
        self.btnFilterPackageRemove = QtGui.QPushButton(self.tabFilterPackages)
        self.btnFilterPackageRemove.setObjectName(_fromUtf8("btnFilterPackageRemove"))
        self.horizontalLayout.addWidget(self.btnFilterPackageRemove)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 0, 1, 2)
        self.lstFilterPackages = QtGui.QTreeView(self.tabFilterPackages)
        self.lstFilterPackages.setIndentation(0)
        self.lstFilterPackages.setObjectName(_fromUtf8("lstFilterPackages"))
        self.gridLayout.addWidget(self.lstFilterPackages, 0, 0, 1, 1)
        self.verticalLayout_3 = QtGui.QVBoxLayout()
        self.verticalLayout_3.setObjectName(_fromUtf8("verticalLayout_3"))
        self.btnFilterPackageMoveUp = QtGui.QToolButton(self.tabFilterPackages)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnFilterPackageMoveUp.sizePolicy().hasHeightForWidth())
        self.btnFilterPackageMoveUp.setSizePolicy(sizePolicy)
        self.btnFilterPackageMoveUp.setObjectName(_fromUtf8("btnFilterPackageMoveUp"))
        self.verticalLayout_3.addWidget(self.btnFilterPackageMoveUp)
        self.btnFilterPackageMoveDown = QtGui.QToolButton(self.tabFilterPackages)
        self.btnFilterPackageMoveDown.setObjectName(_fromUtf8("btnFilterPackageMoveDown"))
        self.verticalLayout_3.addWidget(self.btnFilterPackageMoveDown)
        spacerItem = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem)
        self.gridLayout.addLayout(self.verticalLayout_3, 0, 1, 1, 1)
        self.gbxFilterPackages.addTab(self.tabFilterPackages, _fromUtf8(""))
        self.tabUpdates = QtGui.QWidget()
        self.tabUpdates.setObjectName(_fromUtf8("tabUpdates"))
        self.verticalLayout_4 = QtGui.QVBoxLayout(self.tabUpdates)
        self.verticalLayout_4.setObjectName(_fromUtf8("verticalLayout_4"))
        self.chkUpdates = QtGui.QCheckBox(self.tabUpdates)
        self.chkUpdates.setObjectName(_fromUtf8("chkUpdates"))
        self.verticalLayout_4.addWidget(self.chkUpdates)
        self.chkDevelUpdates = QtGui.QCheckBox(self.tabUpdates)
        self.chkDevelUpdates.setObjectName(_fromUtf8("chkDevelUpdates"))
        self.verticalLayout_4.addWidget(self.chkDevelUpdates)
        self.btnCheckNow = QtGui.QPushButton(self.tabUpdates)
        self.btnCheckNow.setObjectName(_fromUtf8("btnCheckNow"))
        self.verticalLayout_4.addWidget(self.btnCheckNow)
        spacerItem1 = QtGui.QSpacerItem(20, 87, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem1)
        self.gbxFilterPackages.addTab(self.tabUpdates, _fromUtf8(""))
        self.verticalLayout_2.addWidget(self.gbxFilterPackages)
        self.btns = QtGui.QDialogButtonBox(SettingsWidget)
        self.btns.setOrientation(QtCore.Qt.Horizontal)
        self.btns.setStandardButtons(QtGui.QDialogButtonBox.Ok)
        self.btns.setObjectName(_fromUtf8("btns"))
        self.verticalLayout_2.addWidget(self.btns)

        self.retranslateUi(SettingsWidget)
        self.gbxFilterPackages.setCurrentIndex(0)
        QtCore.QObject.connect(self.btns, QtCore.SIGNAL(_fromUtf8("accepted()")), SettingsWidget.accept)
        QtCore.QObject.connect(self.btns, QtCore.SIGNAL(_fromUtf8("rejected()")), SettingsWidget.reject)
        QtCore.QObject.connect(self.chkUpdates, QtCore.SIGNAL(_fromUtf8("toggled(bool)")), self.chkDevelUpdates.setEnabled)
        QtCore.QMetaObject.connectSlotsByName(SettingsWidget)

    def retranslateUi(self, SettingsWidget):
        SettingsWidget.setWindowTitle(_translate("SettingsWidget", "Settings", None))
        self.btnFilterPackageAdd.setText(_translate("SettingsWidget", "Add filter package ...", None))
        self.btnFilterPackageRemove.setText(_translate("SettingsWidget", "Forget package", None))
        self.btnFilterPackageMoveUp.setText(_translate("SettingsWidget", "Up", None))
        self.btnFilterPackageMoveDown.setText(_translate("SettingsWidget", "Down", None))
        self.gbxFilterPackages.setTabText(self.gbxFilterPackages.indexOf(self.tabFilterPackages), _translate("SettingsWidget", "Filter Packages", None))
        self.chkUpdates.setText(_translate("SettingsWidget", "Check for software updates", None))
        self.chkDevelUpdates.setText(_translate("SettingsWidget", "Include development (beta) versions", None))
        self.btnCheckNow.setText(_translate("SettingsWidget", "Check Now", None))
        self.gbxFilterPackages.setTabText(self.gbxFilterPackages.indexOf(self.tabUpdates), _translate("SettingsWidget", "Software Updates", None))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'helpbrowser.ui'
#
# Created: Sun May 31 15:09:06 2015
#      by: PyQt4 UI code generator 4.10.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_HelpBrowser(object):
    def setupUi(self, HelpBrowser):
        HelpBrowser.setObjectName(_fromUtf8("HelpBrowser"))
        HelpBrowser.resize(857, 550)
        self.lyt = QtGui.QGridLayout(HelpBrowser)
        self.lyt.setMargin(0)
        self.lyt.setObjectName(_fromUtf8("lyt"))
        self.tabs = QtGui.QTabWidget(HelpBrowser)
        self.tabs.setDocumentMode(True)
        self.tabs.setTabsClosable(True)
        self.tabs.setMovable(True)
        self.tabs.setObjectName(_fromUtf8("tabs"))
        self.tabHome = QtGui.QWidget()
        self.tabHome.setStyleSheet(_fromUtf8(""))
        self.tabHome.setObjectName(_fromUtf8("tabHome"))
        self.horizontalLayout = QtGui.QHBoxLayout(self.tabHome)
        self.horizontalLayout.setMargin(0)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.scrlFilters = QtGui.QScrollArea(self.tabHome)
        self.scrlFilters.setStyleSheet(_fromUtf8(""))
        self.scrlFilters.setFrameShape(QtGui.QFrame.NoFrame)
        self.scrlFilters.setFrameShadow(QtGui.QFrame.Plain)
        self.scrlFilters.setLineWidth(0)
        self.scrlFilters.setWidgetResizable(True)
        self.scrlFilters.setObjectName(_fromUtf8("scrlFilters"))
        self.wFilters = QtGui.QWidget()
        self.wFilters.setGeometry(QtCore.QRect(0, 0, 857, 527))
        self.wFilters.setStyleSheet(_fromUtf8(""))
        self.wFilters.setObjectName(_fromUtf8("wFilters"))
        self.lytHomeButtons = QtGui.QGridLayout(self.wFilters)
        self.lytHomeButtons.setObjectName(_fromUtf8("lytHomeButtons"))
        self.btnWelcome = QtGui.QPushButton(self.wFilters)
        self.btnWelcome.setProperty("helppath", _fromUtf8("general/welcome"))
        self.btnWelcome.setObjectName(_fromUtf8("btnWelcome"))
        self.lytHomeButtons.addWidget(self.btnWelcome, 0, 0, 1, 1)
        self.btnFilterList = QtGui.QPushButton(self.wFilters)
        self.btnFilterList.setProperty("helppath", _fromUtf8("general/filter-list"))
        self.btnFilterList.setObjectName(_fromUtf8("btnFilterList"))
        self.lytHomeButtons.addWidget(self.btnFilterList, 0, 1, 1, 1)
        self.btnCmdLineHelp = QtGui.QPushButton(self.wFilters)
        self.btnCmdLineHelp.setProperty("helppath", _fromUtf8("general/cmdline"))
        self.btnCmdLineHelp.setObjectName(_fromUtf8("btnCmdLineHelp"))
        self.lytHomeButtons.addWidget(self.btnCmdLineHelp, 1, 0, 1, 1)
        self.btnVersion = QtGui.QPushButton(self.wFilters)
        self.btnVersion.setProperty("helppath", _fromUtf8("general/version"))
        self.btnVersion.setObjectName(_fromUtf8("btnVersion"))
        self.lytHomeButtons.addWidget(self.btnVersion, 1, 1, 1, 1)
        self.scrlFilters.setWidget(self.wFilters)
        self.horizontalLayout.addWidget(self.scrlFilters)
        self.tabs.addTab(self.tabHome, _fromUtf8(""))
        self.lyt.addWidget(self.tabs, 0, 0, 1, 1)

        self.retranslateUi(HelpBrowser)
        self.tabs.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(HelpBrowser)

    def retranslateUi(self, HelpBrowser):
        HelpBrowser.setWindowTitle(_translate("HelpBrowser", "Bibolamazi Help & Reference", None))
        self.btnWelcome.setText(_translate("HelpBrowser", "Welcome - basic Bibolamazi usage", None))
        self.btnWelcome.setProperty("bibolamaziHelpButtonType", _translate("HelpBrowser", "intro", None))
        self.btnFilterList.setText(_translate("HelpBrowser", "Annotated Filter List", None))
        self.btnFilterList.setProperty("bibolamaziHelpButtonType", _translate("HelpBrowser", "intro", None))
        self.btnCmdLineHelp.setText(_translate("HelpBrowser", "Bibolamazi command-line help", None))
        self.btnCmdLineHelp.setProperty("bibolamaziHelpButtonType", _translate("HelpBrowser", "epilog", None))
        self.btnVersion.setText(_translate("HelpBrowser", "Bibolamazi Version", None))
        self.btnVersion.setProperty("bibolamaziHelpButtonType", _translate("HelpBrowser", "epilog", None))
        self.tabs.setTabText(self.tabs.indexOf(self.tabHome), _translate("HelpBrowser", "Home", None))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'filterinstanceeditor.ui'
#
# Created: Sun May 31 15:09:07 2015
#      by: PyQt4 UI code generator 4.10.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_FilterInstanceEditor(object):
    def setupUi(self, FilterInstanceEditor):
        FilterInstanceEditor.setObjectName(_fromUtf8("FilterInstanceEditor"))
        FilterInstanceEditor.resize(368, 361)
        self.gridLayout = QtGui.QGridLayout(FilterInstanceEditor)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.label = QtGui.QLabel(FilterInstanceEditor)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName(_fromUtf8("label"))
        self.horizontalLayout.addWidget(self.label)
        self.cbxFilter = QtGui.QComboBox(FilterInstanceEditor)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(2)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cbxFilter.sizePolicy().hasHeightForWidth())
        self.cbxFilter.setSizePolicy(sizePolicy)
        self.cbxFilter.setEditable(True)
        self.cbxFilter.setInsertPolicy(QtGui.QComboBox.NoInsert)
        self.cbxFilter.setObjectName(_fromUtf8("cbxFilter"))
        self.horizontalLayout.addWidget(self.cbxFilter)
        self.gridLayout.addLayout(self.horizontalLayout, 0, 0, 1, 2)
        self.lstOptions = QtGui.QTreeView(FilterInstanceEditor)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lstOptions.sizePolicy().hasHeightForWidth())
        self.lstOptions.setSizePolicy(sizePolicy)
        self.lstOptions.setVerticalScrollMode(QtGui.QAbstractItemView.ScrollPerPixel)
        self.lstOptions.setHorizontalScrollMode(QtGui.QAbstractItemView.ScrollPerPixel)
        self.lstOptions.setIndentation(0)
        self.lstOptions.setItemsExpandable(False)
        self.lstOptions.setAllColumnsShowFocus(True)
        self.lstOptions.setObjectName(_fromUtf8("lstOptions"))
        self.gridLayout.addWidget(self.lstOptions, 3, 0, 1, 2)
        self.hlytButtons = QtGui.QHBoxLayout()
        self.hlytButtons.setObjectName(_fromUtf8("hlytButtons"))
        self.btnFilterHelp = QtGui.QPushButton(FilterInstanceEditor)
        self.btnFilterHelp.setObjectName(_fromUtf8("btnFilterHelp"))
        self.hlytButtons.addWidget(self.btnFilterHelp)
        self.btnAddFavorite = QtGui.QToolButton(FilterInstanceEditor)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/pic/bookmark.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnAddFavorite.setIcon(icon)
        self.btnAddFavorite.setObjectName(_fromUtf8("btnAddFavorite"))
        self.hlytButtons.addWidget(self.btnAddFavorite)
        self.gridLayout.addLayout(self.hlytButtons, 4, 0, 1, 2)

        self.retranslateUi(FilterInstanceEditor)
        QtCore.QMetaObject.connectSlotsByName(FilterInstanceEditor)

    def retranslateUi(self, FilterInstanceEditor):
        FilterInstanceEditor.setWindowTitle(_translate("FilterInstanceEditor", "Form", None))
        self.label.setText(_translate("FilterInstanceEditor", "Filter:", None))
        self.btnFilterHelp.setText(_translate("FilterInstanceEditor", "Help: Filter Reference", None))
        self.btnAddFavorite.setToolTip(_translate("FilterInstanceEditor", "Add this full command line to your favorites", None))

from . import bibolamazi_res_rc



# -*- coding: utf-8 -*-

################################################################################
#                                                                              #
#   This file is part of the Bibolamazi Project.                               #
#   Copyright (C) 2013 by Philippe Faist                                       #
#   philippe.faist@bluewin.ch                                                  #
#                                                                              #
#   Bibolamazi is free software: you can redistribute it and/or modify         #
#   it under the terms of the GNU General Public License as published by       #
#   the Free Software Foundation, either version 3 of the License, or          #
#   (at your option) any later version.                                        #
#                                                                              #
#   Bibolamazi is distributed in the hope that it will be useful,              #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of             #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              #
#   GNU General Public License for more details.                               #
#                                                                              #
#   You should have received a copy of the GNU General Public License          #
#   along with Bibolamazi.  If not, see <http://www.gnu.org/licenses/>.        #
#                                                                              #
################################################################################

import sys
import os.path
import logging
logger = logging.getLogger(__name__)

from PyQt4.QtCore import *
from PyQt4.QtGui import *

import bibolamazi.init
from bibolamazi.core.bibfilter import factory as filters_factory
from bibolamazi.core import main

from .qtauto.ui_settingswidget import Ui_SettingsWidget


def get_typewriter_font(widget):
    """
    Return a `QFont` with a typewriter font such as Courier. Use some educated guesses for
    choosing a nice font.
    """
    font = widget.font() # default widget font
    font.setStyleHint(QFont.TypeWriter)
    if sys.platform.startswith("darwin"):
        font.setFamily("Menlo")
        font.setPointSize(12)
    else:
        font.setFamily("Monospace")

    return font


# Model to interface a python dictionary with a `set_at()' method, like our
# `filters.PrependOrderedDict`.
#
class MyOrderedDictModel(QAbstractTableModel):
    def __init__(self, dic, parent=None):
        super(MyOrderedDictModel, self).__init__(parent)
        self._dic = dic

    dicChanged = pyqtSignal()

    def rowCount(self, parent):
        if parent.isValid():
            return 0
        return len(self._dic)

    def columnCount(self, parent):
        if parent.isValid():
            return 0
        return 2

    def data(self, index, role=Qt.DisplayRole):

        col = index.column()
        row = index.row()

        if (row < 0 or row >= len(self._dic)):
            return QVariant()

        if role == Qt.BackgroundRole:
            valid = filters_factory.validate_filter_package(str(self._dic.item_at(row)[0]),
                                                            str(self._dic.item_at(row)[1]),
                                                            raise_exception=False)
            if not valid:
                return QVariant(QBrush(QColor(255,200,200)))
            return QVariant()

        if (col == 0):
            # argument name
            if (role == Qt.DisplayRole):
                return QVariant(QString(self._dic.item_at(row)[0]))

            # tool-tip documentation
            if (role == Qt.ToolTipRole):
                return QVariant(QString(self._dic.item_at(row)[0]))

            return QVariant()
        
        if (col == 1):
            # argument value

            if (role == Qt.DisplayRole or role == Qt.EditRole):
                val = self._dic.item_at(row)[1]
                if (val is None):
                    return QVariant()
                return QVariant(QString(str(val)))

            return QVariant()

        return QVariant()


    def headerData(self, section, orientation, role=Qt.DisplayRole):
        if (orientation == Qt.Vertical):
            return QVariant()

        if (section == 0):
            if (role == Qt.DisplayRole):
                return QVariant(QString(u"Filter Package"))
            return QVariant()

        if (section == 1):
            if (role == Qt.DisplayRole):
                return QVariant(QString(u"Path"))
            return QVariant()

        return QVariant()


    def flags(self, index):
        col = index.column()
        row = index.row()

        if col in (0,1,):
            return Qt.ItemIsSelectable | Qt.ItemIsEnabled | Qt.ItemIsEditable

        logger.warning("MyOrderedDictModel.flags(): MyOrderedDictModel.flags(): BAD COLUMN: %d", col)
        return 0
    

    def setData(self, index, value, role=Qt.EditRole):
        
        col = index.column()
        row = index.row()

        if col not in (0,1):
            return False

        if (role != Qt.EditRole):
            return False

        if (row < 0 or row >= len(filters_factory.filterpath)):
            return False

        value = value.toPyObject()
        
        if (isinstance(value, QString)):
            value = unicode(value); # make sure we're dealing with Python strings and not Qt strings

        logger.debug("Got value: %r", value)

        if col == 0:
            self._dic.set_at(row, value, self._dic.item_at(row)[1])
        if col == 1:
            self._dic.set_at(row, self._dic.item_at(row)[0], value)

        self.dataChanged.emit(index, index)

        self.dicChanged.emit()

        return True





def setup_filterpackages_from_settings(s):

    s.beginGroup("BibolamaziCore")

    fpstr = str(s.value("filterpath").toString())

    for fp in reversed(fpstr.split(os.pathsep)):
        # if we had 'filters=' from some old version, then replace that by 'bibolamazi.filters='
        if fp == 'filters=':
            fp = 'bibolamazi.filters='

        main.setup_filterpackage_from_argstr(fp)

    s.endGroup()




class SettingsWidget(QDialog):
    def __init__(self, swu_interface, swu_sourcefilter_devel, mainwin=None):
        super(SettingsWidget, self).__init__(parent=mainwin)

        logger.debug("swu_interface=%r, swu_sourcefilter_devel=%r", swu_interface, swu_sourcefilter_devel)

        self.swu_interface = swu_interface
        self.swu_sourcefilter_devel = swu_sourcefilter_devel

        self.mainwin = mainwin

        self.ui = Ui_SettingsWidget()
        self.ui.setupUi(self)

        # filter packages

        self.fpmodel = MyOrderedDictModel(filters_factory.filterpath)
        self.ui.lstFilterPackages.setModel(self.fpmodel)

        self.ui.lstFilterPackages.selectionModel().selectionChanged.connect(
            self.slot_lstFilterPackages_selectionChanged
            )
        self.slot_lstFilterPackages_selectionChanged()
        self.fpmodel.dicChanged.connect(self.save_settings)

        # software updates

        if (self.swu_interface is None or self.swu_sourcefilter_devel is None):
            self.ui.tabUpdates.setEnabled(False)
        else:
            self.ui.tabUpdates.setEnabled(True)
            self.ui.chkUpdates.setChecked(self.swu_interface.checkForUpdatesEnabled())
            self.ui.chkDevelUpdates.setChecked(self.swu_sourcefilter_devel.includeDevelReleases())
            self.swu_interface.checkForUpdatesEnabledChanged.connect(self.ui.chkUpdates.setChecked)


    def filterpackages_selected_rows(self):
        idxlist = self.ui.lstFilterPackages.selectedIndexes()
        rows = set([i.row() for i in idxlist])
        return list(rows)

    @pyqtSlot()
    def slot_lstFilterPackages_selectionChanged(self):
        if not self.ui.lstFilterPackages.selectionModel().hasSelection():
            self.ui.btnFilterPackageRemove.setEnabled(False)
            self.ui.btnFilterPackageMoveUp.setEnabled(False)
            self.ui.btnFilterPackageMoveDown.setEnabled(False)
            return

        self.ui.btnFilterPackageRemove.setEnabled(True)
        idx = self.filterpackages_selected_rows()
        if len(idx) == 1:
            self.ui.btnFilterPackageMoveUp.setEnabled(idx[0] > 0)
            self.ui.btnFilterPackageMoveDown.setEnabled(idx[0] < len(filters_factory.filterpath)-1)
        else:
            self.ui.btnFilterPackageMoveUp.setEnabled(False)
            self.ui.btnFilterPackageMoveDown.setEnabled(False)
        

    @pyqtSlot()
    def on_btnFilterPackageAdd_clicked(self):

        thedir = str(QFileDialog.getExistingDirectory(self, "Locate Filter Package", QString()))

        thekey = os.path.basename(thedir)
        thedir = os.path.dirname(thedir)

        if thekey in filters_factory.filterpath:
            QMessageBox.critical(self, "There is already an existing filter package `%s'!", thekey)

        if thekey and thedir:
            filters_factory.filterpath[thekey] = str(thedir)
            self.ui.lstFilterPackages.reset();
            self.save_settings()

    @pyqtSlot()
    def on_btnFilterPackageRemove_clicked(self):
        idxlist = self.filterpackages_selected_rows()
        if not len(idxlist):
            return

        for idx in idxlist:
            yn = QMessageBox.question(self, "Remove filter package?",
                                      ("Unset filter package %s? The files will not be removed, "
                                      "they will just be ignored by bibolamazi.") % (
                                          filters_factory.filterpath.item_at(idx)[0]
                                          ),
                                      QMessageBox.Yes|QMessageBox.Cancel, QMessageBox.Cancel)
            if yn != QMessageBox.Yes:
                continue
            
            del filters_factory.filterpath[filters_factory.filterpath.item_at(idx)[0]]
            self.ui.lstFilterPackages.reset()
            self.save_settings()

    @pyqtSlot()
    def on_btnFilterPackageMoveUp_clicked(self):
        idxlist = self.filterpackages_selected_rows()
        if len(idxlist) != 1:
            return
        row = idxlist[0]

        if row == 0:
            return

        fpitems = filters_factory.filterpath.items()
        filters_factory.filterpath.clear()
        filters_factory.filterpath.update(reversed( fpitems[:row-1] + [fpitems[row], fpitems[row-1]] + fpitems[row+1:] ))
        self.ui.lstFilterPackages.reset()
        self.save_settings()
        
    @pyqtSlot()
    def on_btnFilterPackageMoveDown_clicked(self):
        idxlist = self.filterpackages_selected_rows()
        if len(idxlist) != 1:
            return
        row = idxlist[0]

        if row >= len(filters_factory.filterpath)-1:
            return

        fpitems = filters_factory.filterpath.items()
        filters_factory.filterpath.clear()
        filters_factory.filterpath.update(reversed( fpitems[:row] + [fpitems[row+1], fpitems[row]] + fpitems[row+2:] ))
        self.ui.lstFilterPackages.reset()
        self.save_settings()


    @pyqtSlot(bool)
    def on_chkUpdates_toggled(self, val):
        if self.swu_interface:
            self.swu_interface.setCheckForUpdatesEnabled(val)

    @pyqtSlot(bool)
    def on_chkDevelUpdates_toggled(self, val):
        if self.swu_sourcefilter_devel:
            self.swu_sourcefilter_devel.setIncludeDevelReleases(val)

    @pyqtSlot()
    def on_btnCheckNow_clicked(self):
        self.mainwin.doCheckForUpdates()



    @pyqtSlot()
    def save_settings(self):

        s = QSettings()

        s.beginGroup("BibolamaziCore")

        s.setValue("filterpath",
                   QVariant(QString(os.pathsep.join(( "%s=%s"%(k,v if v else "")
                                                      for k,v in filters_factory.filterpath.items() ))))
                   )
        # reset the filter cache.
        filters_factory.reset_filters_cache()

        s.endGroup()
        s.sync()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mainwidget.ui'
#
# Created: Sun May 31 15:09:06 2015
#      by: PyQt4 UI code generator 4.10.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_MainWidget(object):
    def setupUi(self, MainWidget):
        MainWidget.setObjectName(_fromUtf8("MainWidget"))
        MainWidget.resize(491, 545)
        self.verticalLayout_3 = QtGui.QVBoxLayout(MainWidget)
        self.verticalLayout_3.setObjectName(_fromUtf8("verticalLayout_3"))
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        spacerItem = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.lblMain = QtGui.QLabel(MainWidget)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblMain.sizePolicy().hasHeightForWidth())
        self.lblMain.setSizePolicy(sizePolicy)
        self.lblMain.setPixmap(QtGui.QPixmap(_fromUtf8(":/pic/bibolamazi.png")))
        self.lblMain.setObjectName(_fromUtf8("lblMain"))
        self.horizontalLayout.addWidget(self.lblMain)
        spacerItem1 = QtGui.QSpacerItem(40, 20, QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout_3.addLayout(self.horizontalLayout)
        self.groupBox = QtGui.QGroupBox(MainWidget)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.verticalLayout = QtGui.QVBoxLayout(self.groupBox)
        self.verticalLayout.setSpacing(20)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.btnOpenFile = QtGui.QPushButton(self.groupBox)
        self.btnOpenFile.setObjectName(_fromUtf8("btnOpenFile"))
        self.verticalLayout.addWidget(self.btnOpenFile)
        self.btnNewFile = QtGui.QPushButton(self.groupBox)
        self.btnNewFile.setObjectName(_fromUtf8("btnNewFile"))
        self.verticalLayout.addWidget(self.btnNewFile)
        self.verticalLayout_3.addWidget(self.groupBox)
        self.groupBox_2 = QtGui.QGroupBox(MainWidget)
        self.groupBox_2.setObjectName(_fromUtf8("groupBox_2"))
        self.verticalLayout_2 = QtGui.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_2.setSpacing(20)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.btnSettings = QtGui.QPushButton(self.groupBox_2)
        self.btnSettings.setObjectName(_fromUtf8("btnSettings"))
        self.verticalLayout_2.addWidget(self.btnSettings)
        self.btnHelp = QtGui.QPushButton(self.groupBox_2)
        self.btnHelp.setObjectName(_fromUtf8("btnHelp"))
        self.verticalLayout_2.addWidget(self.btnHelp)
        self.verticalLayout_3.addWidget(self.groupBox_2)
        spacerItem2 = QtGui.QSpacerItem(20, 40, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem2)
        self.btnQuit = QtGui.QPushButton(MainWidget)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/pic/closehide.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnQuit.setIcon(icon)
        self.btnQuit.setObjectName(_fromUtf8("btnQuit"))
        self.verticalLayout_3.addWidget(self.btnQuit)

        self.retranslateUi(MainWidget)
        QtCore.QMetaObject.connectSlotsByName(MainWidget)

    def retranslateUi(self, MainWidget):
        MainWidget.setWindowTitle(_translate("MainWidget", "Bibolamazi", None))
        self.groupBox.setTitle(_translate("MainWidget", "Open bibolamazi file or create new bibolamazi file", None))
        self.btnOpenFile.setText(_translate("MainWidget", "Open Bibolamazi File ...", None))
        self.btnNewFile.setText(_translate("MainWidget", "Create New Bibolamazi File ...", None))
        self.groupBox_2.setTitle(_translate("MainWidget", "Settings && Help", None))
        self.btnSettings.setText(_translate("MainWidget", "Settings", None))
        self.btnHelp.setText(_translate("MainWidget", "Help && Reference Browser", None))
        self.btnQuit.setText(_translate("MainWidget", "Quit", None))

from . import bibolamazi_res_rc




# -*- coding: utf-8 -*-

################################################################################
#                                                                              #
#   This file is part of the Bibolamazi Project.                               #
#   Copyright (C) 2013 by Philippe Faist                                       #
#   philippe.faist@bluewin.ch                                                  #
#                                                                              #
#   Bibolamazi is free software: you can redistribute it and/or modify         #
#   it under the terms of the GNU General Public License as published by       #
#   the Free Software Foundation, either version 3 of the License, or          #
#   (at your option) any later version.                                        #
#                                                                              #
#   Bibolamazi is distributed in the hope that it will be useful,              #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of             #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              #
#   GNU General Public License for more details.                               #
#                                                                              #
#   You should have received a copy of the GNU General Public License          #
#   along with Bibolamazi.  If not, see <http://www.gnu.org/licenses/>.        #
#                                                                              #
################################################################################


import os
import os.path
import logging
logger = logging.getLogger(__name__)


from PyQt4.QtCore import *
from PyQt4.QtGui import *

import bibolamazi.init
from .qtauto.ui_sourcelisteditor import Ui_SourceListEditor


class SourceListEditor(QWidget):
    def __init__(self, parent):
        super(SourceListEditor, self).__init__(parent)

        self.ui = Ui_SourceListEditor()
        self.ui.setupUi(self)
        
        self.ui.btnAddFavorite.clicked.connect(self.requestAddToFavorites)

        QObject.connect(self.ui.lstSources.model(), SIGNAL('layoutChanged()'), self.update_stuff_moved)

        self._is_updating = False

        self._ref_dir = None


    sourceListChanged = pyqtSignal('QStringList')

    requestAddToFavorites = pyqtSignal()
    

    def sourceList(self):
        return [str(self.ui.lstSources.item(i).text())  for i in xrange(self.ui.lstSources.count())]


    def setRefDir(self, refdir):
        """Sets the \"reference\" directory, which is the directory in which the bibolamazi
        file being edited resides. This is used to decide on whether to refer to a file with
        an absolute or a relative path.
        """
        self._ref_dir = refdir;
        

    @pyqtSlot(QStringList, bool)
    @pyqtSlot(QStringList)
    def setSourceList(self, sourcelist, noemit=False):
        sourcelist = [str(x) for x in list(sourcelist)];
        
        # don't reset source list if it's the same. In particular, don't emit the changed signal.
        if (sourcelist == self.sourceList()):
            return
        
        self._is_updating = True
        self.ui.lstSources.clear()
        for src in sourcelist:
            self.ui.lstSources.addItem(src)

        # refresh current row
        self.on_lstSources_currentRowChanged(self.ui.lstSources.currentRow())

        self._is_updating = False
        if (not noemit):
            self.emitSourceListChanged()


    @pyqtSlot()
    def emitSourceListChanged(self):
        if (self._is_updating):
            return
        logger.debug("emitting sourceListChanged()!")
        self.sourceListChanged.emit(QStringList(self.sourceList()))


    @pyqtSlot()
    def on_btnAddSource_clicked(self):
        self.ui.lstSources.addItem("")
        self.ui.lstSources.setCurrentRow(self.ui.lstSources.count()-1)
        self.ui.txtFile.setFocus()
        self.emitSourceListChanged()

    @pyqtSlot()
    def on_btnRemoveSource_clicked(self):
        row = self.ui.lstSources.currentRow()
        if (row < 0):
            logger.debug("No row selected")
            return

        logger.debug('removing row %d', row)
        item = self.ui.lstSources.takeItem(row)
        # ###TODO: FIXME: delete item?!?

        self.emitSourceListChanged()

    @pyqtSlot()
    def update_stuff_moved(self):
        # user moved stuff around
        logger.debug("Stuff moved around!")

        self.emitSourceListChanged()
        
    @pyqtSlot('QListWidgetItem*')
    def on_lstSources_itemDoubleClicked(self, item):
        logger.debug('double-clicked!!')
        self.ui.txtFile.setFocus()

    @pyqtSlot(int)
    def on_lstSources_currentRowChanged(self, row):
        logger.debug("current row changed.. row=%d", row)
        if (self.ui.lstSources.count() == 0  or  row < 0):
            self.ui.txtFile.setText("")
            self.ui.btnRemoveSource.setEnabled(False)
            self.ui.gbxEditSource.setEnabled(False)
        else:
            self.ui.btnRemoveSource.setEnabled(True)
            self.ui.gbxEditSource.setEnabled(True)
            self.ui.txtFile.setText(self.ui.lstSources.item(row).text())


    @pyqtSlot(QString)
    def on_txtFile_textChanged(self, text):
        row = self.ui.lstSources.currentRow()
        if (row < 0):
            logger.debug("No row selected")
            return

        item = self.ui.lstSources.item(row);
        if (item.text() != text):
            item.setText(text)
            self.emitSourceListChanged()

    @pyqtSlot()
    def on_btnBrowse_clicked(self):
        row = self.ui.lstSources.currentRow()
        if (row < 0):
            logger.debug("No row selected")
            return
        
        fname = str(QFileDialog.getOpenFileName(self, 'Select BibTeX File', QString(),
                                                'BibTeX Files (*.bib);;All Files (*)'))
        logger.debug("fname=%r.", fname)
        if (not fname):
            return

        # decide on whether to refer to file in absolute or relative fashion
        if (self._ref_dir):
            relpath = os.path.relpath(os.path.realpath(fname), os.path.realpath(self._ref_dir))
            if '..' in relpath.split(os.sep) or '..' in relpath.split(os.altsep):
                # out of scope, so use absolute.
                pass
            else:
                # can easily be referred to by relative path (in same or sub directory), use relative.
                fname = relpath

        self.ui.txtFile.setText(fname)
        



#!/usr/bin/env python
# -*- coding: utf-8 -*-

################################################################################
#                                                                              #
#   This file is part of the Bibolamazi Project.                               #
#   Copyright (C) 2013 by Philippe Faist                                       #
#   philippe.faist@bluewin.ch                                                  #
#                                                                              #
#   Bibolamazi is free software: you can redistribute it and/or modify         #
#   it under the terms of the GNU General Public License as published by       #
#   the Free Software Foundation, either version 3 of the License, or          #
#   (at your option) any later version.                                        #
#                                                                              #
#   Bibolamazi is distributed in the hope that it will be useful,              #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of             #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              #
#   GNU General Public License for more details.                               #
#                                                                              #
#   You should have received a copy of the GNU General Public License          #
#   along with Bibolamazi.  If not, see <http://www.gnu.org/licenses/>.        #
#                                                                              #
################################################################################


import sys
import os
import os.path
import re
import logging
import subprocess
import datetime

from PyQt4.QtCore import *
from PyQt4.QtGui import *

import bibolamazi.init

# set up basic logging
from bibolamazi.core import blogger

from bibolamazi.core import bibolamazifile
from bibolamazi.core import main
from bibolamazi.core.butils import BibolamaziError
from bibolamazi.core.bibfilter import factory as filters_factory
from bibolamazi.core import version as bibolamaziversion

import openbibfile
import helpbrowser
import settingswidget

from .favorites import FavoriteCmdsList

from .qtauto.ui_mainwidget import Ui_MainWidget


logger = logging.getLogger(__name__)



class BibolamaziApplication(QApplication):
    def __init__(self):
        self.main_widget = None
        
        super(BibolamaziApplication, self).__init__(sys.argv)
        
        self.setWindowIcon(QIcon(':/pic/bibolamazi_icon.png'))
        self.setApplicationName('Bibolamazi')
        self.setApplicationVersion(bibolamaziversion.version_str)
        self.setOrganizationDomain('org.bibolamazi')
        self.setOrganizationName('Bibolamazi Project')

        # before main widget, so that main widget can create & connect relevant menu items
        setup_software_updater()

        # create & setup main widget
        self.main_widget = MainWidget()

        self.main_widget.show()
        self.main_widget.raise_()

    def event(self, event):
        if (event.type() == QEvent.FileOpen):
            logger.info("Opening file %s", event.file())
            # request to open file
            if (self.main_widget is None):
                logger.error("ERROR: CAN'T OPEN FILE: MAIN WIDGET IS NONE!")
            else:
                self.main_widget.openFile(event.file())
            return True
        return super(BibolamaziApplication, self).event(event)



def find_retina_resolution():
    try:
        output = subprocess.check_output(['system_profiler', 'SPDisplaysDataType'])
    except Exception as e:
        logger.info("Couldn't check for retina display: %s", e)
        return
    logger.debug("Got display information:\n%s", output)
    m = re.search(r'Retina:\s*(?P<answer>Yes|No)', output, flags=re.IGNORECASE)
    if not m:
        return None
    if m.group('answer').lower() != 'yes':
        return None
    m2 = re.search(r'Resolution:\s*(?P<resX>\d+)\s*x\s*(?P<resY>\d+)', output, flags=re.IGNORECASE)
    if not m:
        logger.info("Couldn't find resolution information for retina display.")
        return None
    return (int(m2.group('resX')), int(m2.group('resY')))

class MainWidget(QWidget):
    def __init__(self):
        super(MainWidget, self).__init__()

        self.ui = Ui_MainWidget()
        self.ui.setupUi(self)

        # set up nice vector graphics on retina displays
        if sys.platform.startswith("darwin") and QT_VERSION_STR.startswith("4.8."):
            # use high-res SVG for retina displays
            retinaresolution = find_retina_resolution()
            if retinaresolution is not None:
                mydesktop = QApplication.desktop()
                # seems that myratio is not reliable (I get 1.77777..), so just use 2x
                #myratio = float(retinaresolution[0]) / mydesktop.width()
                myratio = 2
                print 'myratio=', myratio
                #if myratio > 1.01:
                self.mypict = QPicture()
                mypaint = QPainter(self.mypict)
                self.myicon = QIcon(":/pic/bibolamazi.svg")
                mysize = QSize(375, 150)
                mypaint.drawPixmap(QRect(QPoint(0,0),mysize), self.myicon.pixmap(myratio*mysize))
                self.ui.lblMain.setPicture(self.mypict)

        self.openbibfiles = []

        self.helpbrowser = None

        self.settingswidget = None

        self.favoriteCmdsList = FavoriteCmdsList(parent=self)
        self.favoriteCmdsList.loadFromSettings(QSettings())

        self.menubar = None
        self.shortcuts = []

        self.upd_checkenabled_action = None
        self.upd_checknow_action = None
        if (swu_interface is not None):
            self.upd_checkenabled_action = QAction("Regularly Check for Updates", self)
            self.upd_checkenabled_action.setCheckable(True)
            self.upd_checkenabled_action.setChecked(swu_interface.checkForUpdatesEnabled())
            self.upd_checkenabled_action.toggled.connect(swu_interface.setCheckForUpdatesEnabled)
            swu_interface.checkForUpdatesEnabledChanged.connect(self.upd_checkenabled_action.setChecked)
            #self.upd_checkenabled_action.toggled.connect(
            #    lambda value:
            #    QMessageBox.information(self, 'Software Updates',
            #                            'Software update checks have been turned %s.' %('on' if value else 'off'))
            #    )
            self.upd_checknow_action = QAction("Check for Updates Now", self)
            self.upd_checknow_action.triggered.connect(self.doCheckForUpdates)

        if (sys.platform.startswith('darwin')):
            # Mac OS X
            self.menubar = QMenuBar(None)
            filemenu = self.menubar.addMenu("File")
            filemenu.addAction("New", self, SLOT('on_btnNewFile_clicked()'),
                               QKeySequence("Ctrl+N"))
            filemenu.addAction("Open", self, SLOT('on_btnOpenFile_clicked()'),
                               QKeySequence("Ctrl+O"))
            filemenu.addAction("Settings", self, SLOT('on_btnSettings_clicked()'))
            if (self.upd_checkenabled_action):
                filemenu.addSeparator()
                filemenu.addAction(self.upd_checkenabled_action)
            if (self.upd_checknow_action):
                filemenu.addAction(self.upd_checknow_action)
            helpmenu = self.menubar.addMenu("Help")
            helpmenu.addAction("Open Help && Reference Browser", self, SLOT('on_btnHelp_clicked()'),
                               QKeySequence("Ctrl+R"))
        else:
            self.shortcuts += [
                (QAction("New", self), "Ctrl+N", self.on_btnNewFile_clicked),
                (QAction("Open", self), "Ctrl+O", self.on_btnOpenFile_clicked),
                (QAction("Help", self), "Ctrl+R", self.on_btnHelp_clicked),
                (QAction("Quit", self), "Ctrl+Q", self.on_btnQuit_clicked),
                (QAction("Settings", self), "Ctrl+P", self.on_btnSettings_clicked),
                #
                # PyQt Bug: these shortcuts cause segfaults!! workaround: use QAction's instead.
                #
                #QShortcut(QKeySequence('Ctrl+N'), self, self.on_btnNewFile_clicked, self.on_btnNewFile_clicked,
                #          Qt.ApplicationShortcut),
                #QShortcut(QKeySequence('Ctrl+O'), self, self.on_btnOpenFile_clicked, self.on_btnOpenFile_clicked,
                #          Qt.ApplicationShortcut),
                #QShortcut(QKeySequence('Ctrl+R'), self, self.on_btnHelp_clicked, self.on_btnHelp_clicked,
                #          Qt.ApplicationShortcut),
                #QShortcut(QKeySequence('Ctrl+Q'), self, self.on_btnQuit_clicked, self.on_btnQuit_clicked,
                #          Qt.ApplicationShortcut),
                ]
            if (self.upd_checkenabled_action):
                self.shortcuts += [
                    #(self.upd_checkenabled_action, "Ctrl+U", None)
                    ]
            if (self.upd_checknow_action):
                self.shortcuts += [
                    (self.upd_checknow_action, "Ctrl+U", None)
                    ]
            # now, setup the shortcuts.
            for (a, key, slot) in self.shortcuts:
                #print 'adding action with key %s' %(key)
                a.setShortcut(QKeySequence(key))
                if (slot is not None):
                    a.triggered.connect(slot)
                a.setShortcutContext(Qt.ApplicationShortcut)
                self.addAction(a)

        self.setWindowIcon(QIcon(':/pic/bibolamazi_icon.png'))


    def doCheckForUpdates(self):
        if swu_interface is not None:
            ret = swu_interface.do_check_for_updates()
            if ret is None:
                pass # no checking for updates
            elif ret is False:
                # no new updates
                QMessageBox.information(self, "Software Update Check",
                                        "There are no new updates.")
            elif isinstance(ret, tuple):
                if len(ret) == 3:
                    QMessageBox.critical(self, "Error: Software Update Check",
                                         ret[2]);
                    return
                # if ret[0]==True, then update installed but not restarted.
                # if ret[0]==False, then udpate exists, but not installed.
                return
                    
        
    def openFile(self, fname):
        w = openbibfile.OpenBibFile()
        w.setFavoriteCmdsList(self.favoriteCmdsList)
        w.setOpenFile(fname)
        w.show()
        w.raise_()
        w.fileClosed.connect(self.bibFileClosed)
        self.openbibfiles.append(w)

        w.requestHelpTopic.connect(self.openHelpTopic)


    @pyqtSlot(QString)
    def openHelpTopic(self, path):
        self.on_btnHelp_clicked()
        self.helpbrowser.openHelpTopic(path)


    @pyqtSlot()
    def on_btnOpenFile_clicked(self):
        fname = str(QFileDialog.getOpenFileName(self, 'Open Bibolamazi File', QString(),
                                                'Bibolamazi Files (*.bibolamazi.bib);;All Files (*)'))
        if (fname):
            self.openFile(fname)
        
    @pyqtSlot()
    def on_btnNewFile_clicked(self):
        saveFileDialog = QFileDialog(self, "Create Bibolamazi File", QString(),
                                     "Bibolamazi Files (*.bibolamazi.bib);;All Files (*)");
        if sys.platform.startswith('darwin'):
            # NOTE: BUG: OS X' file selection dialog is so stupid it won't understand the
            # .bibolamazi.bib extension and will force some silly warning dialong and mess up
            # the extension completely. So use Qt's file dialog.
            saveFileDialog.setOptions(QFileDialog.DontUseNativeDialog)
        
        saveFileDialog.setDefaultSuffix("bibolamazi.bib")
        saveFileDialog.setAcceptMode(QFileDialog.AcceptSave)
        saveFileDialog.setFileMode(QFileDialog.AnyFile)
        if not saveFileDialog.exec_():
            return
        
        newfilename = [unicode(x) for x in saveFileDialog.selectedFiles()][0]
            
        if (not newfilename):
            # cancelled
            return

        if (os.path.exists(newfilename)):
            QMessageBox.critical(self, "File Exists",
                                 "Cowardly refusing to overwrite existing file `%s'. Please remove it first."
                                 %(newfilename))
            return

        try:
            bfile = bibolamazifile.BibolamaziFile(newfilename, create=True);
            bfile.saveToFile();
        except Exception as e:
            QMessageBox.critical(self, "Error", "Error: Can't create file: %s"%(e))
            return

        self.openFile(newfilename)

    @pyqtSlot()
    def on_btnHelp_clicked(self):
        if (self.helpbrowser is None):
            self.helpbrowser = helpbrowser.HelpBrowser()
        self.helpbrowser.show()
        self.helpbrowser.raise_()

    @pyqtSlot()
    def on_btnQuit_clicked(self):
        self.close()


    @pyqtSlot()
    def on_btnSettings_clicked(self):
        if self.settingswidget is None:
            self.settingswidget = settingswidget.SettingsWidget(swu_interface=swu_interface,
                                                                swu_sourcefilter_devel=swu_sourcefilter_devel,
                                                                mainwin=self)
        self.settingswidget.show()
        self.settingswidget.raise_()

    @pyqtSlot()
    def bibFileClosed(self):
        sender = self.sender()
        if (not sender in self.openbibfiles):
            logger.warning("Widget sender of fileClosed() not in our openbibfiles list!!")
            return
        logger.debug("file is closed.")
        self.openbibfiles.remove(sender)

    def closeEvent(self, event):
        logger.debug("Close!!")

        for w in self.openbibfiles:
            ans = w.close()
            if not ans:
                # if the widget cancels the close, then abort
                event.ignore()
                return

        if (self.helpbrowser):
            self.helpbrowser.close()

        self.favoriteCmdsList.saveToSettings(QSettings())

        super(MainWidget, self).closeEvent(event)


swu_updater = None
swu_interface = None
swu_source = None
swu_sourcefilter_devel = None


def setup_software_updater():
    if (not hasattr(sys, '_MEIPASS')):
        # not pyinstaller-packaged
        return

    global swu_updater, swu_interface, swu_source, swu_sourcefilter_devel

    import logging
    from updater4pyi import upd_core, upd_source, upd_iface, upd_log
    from updater4pyi.upd_source import relpattern, RELTYPE_BUNDLE_ARCHIVE, RELTYPE_EXE
    from updater4pyi.upd_iface_pyqt4 import UpdatePyQt4Interface

    # updater4pyi's logger will use our main logger anyway.
    #upd_log.setup_logger(logging.DEBUG)

    # DEBUG:
    #upd_iface.DEFAULT_INIT_CHECK_DELAY = datetime.timedelta(days=0, seconds=3, microseconds=0) # seconds
    #upd_iface.DEFAULT_CHECK_INTERVAL = datetime.timedelta(days=0, seconds=10, microseconds=0) # seconds

    swu_source = upd_source.UpdateGithubReleasesSource('phfaist/bibolamazi')

    swu_sourcefilter_devel = upd_source.UpdateSourceDevelopmentReleasesFilter(False);
    swu_source.add_release_filter(swu_sourcefilter_devel)

    swu_updater = upd_core.Updater(current_version=bibolamaziversion.version_str, #'0.9', ## DEBUG!!! 
                                   update_source=swu_source)

    swu_interface = UpdatePyQt4Interface(swu_updater, progname='Bibolamazi', ask_before_checking=True,
                                         parent=QApplication.instance())
    swu_interface.start()



def run_main():

    blogger.setup_simple_console_logging()

    # default level: set to root logger
    #logging.getLogger().setLevel(blogger.LONGDEBUG)
    logging.getLogger().setLevel(logging.DEBUG)

    # ## Seems we still need this for pyinstaller, I'm not sure why....
    #
    # load precompiled filters, if we've got any
    try:
        import bibolamazi_compiled_filter_list as pc
        filters_factory.load_precompiled_filters('bibolamazi.filters', dict([
            (fname, pc.__dict__[fname])  for fname in pc.filter_list
            ]))
    except ImportError:
        pass


    logger.debug("starting application")

    app = BibolamaziApplication();

    try:
        # load filter packages from environment ...
        main.setup_filterpackages_from_env()
        # ... and from settings.
        settingswidget.setup_filterpackages_from_settings(QSettings())
    except (filters_factory.NoSuchFilter, filters_factory.NoSuchFilterPackage, BibolamaziError):
        QMessageBox.warning(None, "Filter packages error",
                            "An error was detected in the filter packages configuration. "
                            "Please edit your settings.")
        pass

    args = app.arguments();
    _rxscript = re.compile('\.(py[co]?|exe)$', flags=re.IGNORECASE);
    for k in xrange(1,len(args)): # skip program name == argv[0]
        fn = str(args[k])
        if (_rxscript.search(fn)):
            # our own script, bug on windows?
            logger.debug("skipping own arg: %s", fn)
            continue
        
        logger.debug("opening arg: %s", fn)
        app.main_widget.openFile(fn);

    sys.exit(app.exec_())
    

if __name__ == '__main__':

    run_main()




# -*- coding: utf-8 -*-

################################################################################
#                                                                              #
#   This file is part of the Bibolamazi Project.                               #
#   Copyright (C) 2013 by Philippe Faist                                       #
#   philippe.faist@bluewin.ch                                                  #
#                                                                              #
#   Bibolamazi is free software: you can redistribute it and/or modify         #
#   it under the terms of the GNU General Public License as published by       #
#   the Free Software Foundation, either version 3 of the License, or          #
#   (at your option) any later version.                                        #
#                                                                              #
#   Bibolamazi is distributed in the hope that it will be useful,              #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of             #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the              #
#   GNU General Public License for more details.                               #
#                                                                              #
#   You should have received a copy of the GNU General Public License          #
#   along with Bibolamazi.  If not, see <http://www.gnu.org/licenses/>.        #
#                                                                              #
################################################################################

import sys
import logging

import bibolamazi.init
from bibolamazi.core import main as bibolamazimain
from bibolamazi.core import blogger
from bibolamazi.core.blogger import logger
from bibolamazi.core import butils
from bibolamazi.core import argparseactions
from bibolamazi.core.bibfilter import factory as filters_factory

from PyQt4.QtCore import *
from PyQt4.QtGui import *

from . import filterinstanceeditor
from . import settingswidget

from .qtauto.ui_helpbrowser import Ui_HelpBrowser

logger = logging.getLogger(__name__)




_HOME_TAB_STYLESHEET = '''
QWidget {
}

#wFilters {
    padding: 10px 50px 10px 50px;
    background-color: white;
}


QPushButton {
    color: rgba(255,255,255,255);
    padding: 8px 2px;
}
QPushButton {
    /* light blue */
    background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 rgba(0, 113, 188, 255), stop:0.9 rgba(64, 91, 110, 255));
}
QPushButton[bibolamaziHelpButtonType="filter"] {
    /* bordeau */
    background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 rgba(158, 0, 93, 255), stop:0.9 rgba(82, 55, 71, 255));
}
QPushButton[bibolamaziHelpButtonType="intro"] {
    /* dark blue */
	background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 rgba(27, 20, 100, 255), stop:0.9 rgba(77, 75, 99, 255));
}
'''




class HelpBrowser(QWidget):
    def __init__(self):
        super(HelpBrowser, self).__init__()

        self.ui = Ui_HelpBrowser()
        self.ui.setupUi(self)

        QObject.connect(self.ui.tabs, SIGNAL('tabCloseRequested(int)'), self.closeTab)

        self.filterButtons = []

        self.openTabs = []


        # home buttons
        # ------------

        self.ui.lytHomeButtons.setContentsMargins(60, 30, 60, 30)

        vspc1 = QSpacerItem(20, 5, QSizePolicy.Fixed, QSizePolicy.Fixed)
        self.ui.lytHomeButtons.addItem(vspc1, 1, 0)

        offsetlineno = 2 # start at 1, because the first row has the 'welcome' and 'filter list' buttons.
        n = 0 # count how many filters we've treated already.
        ncols = 2

        for filt in filterinstanceeditor.get_filter_list():
            fbutton = QPushButton('%s' % (filt), self)
            fbutton.setProperty('helppath', 'filters/%s' %(filt))
            fbutton.setProperty('bibolamaziHelpButtonType', 'filter')
            fbutton.setToolTip(filters_factory.get_filter_class(filt).getHelpDescription())
            self.ui.lytHomeButtons.addWidget(fbutton, offsetlineno + int(n / ncols), n % ncols)
            n += 1

            QObject.connect(fbutton, SIGNAL('clicked()'), self.openHelpTopicBySender)

        newrow = None
        if n % ncols == 0:
            newrow = offsetlineno + n / ncols
        else:
            newrow = offsetlineno + (1 + int(n / ncols))

        vspc3 = QSpacerItem(20, 5, QSizePolicy.Fixed, QSizePolicy.Fixed)
        self.ui.lytHomeButtons.addItem(vspc3, newrow, 0)

        self.ui.lytHomeButtons.addWidget(self.ui.btnCmdLineHelp, newrow+1, 0)
        self.ui.lytHomeButtons.addWidget(self.ui.btnVersion, newrow+1, 1)
        vspc2 = QSpacerItem(20, 1, QSizePolicy.Minimum, QSizePolicy.Expanding)
        self.ui.lytHomeButtons.addItem(vspc2, newrow+2, 0)

        # avoid python garbage collection
        self.vspcButtons = [vspc1, vspc2]

        self.ui.tabHome.setStyleSheet(_HOME_TAB_STYLESHEET)

        static_help_btns = [ self.ui.btnWelcome,
                             self.ui.btnVersion,
                             self.ui.btnFilterList,
                             self.ui.btnCmdLineHelp
                             ]
        for btn in static_help_btns:
            QObject.connect(btn, SIGNAL('clicked()'), self.openHelpTopicBySender)

        self.shortcuts = [
            QShortcut(QKeySequence('Ctrl+W'), self, self.closeCurrentTab, self.closeCurrentTab),
            ]


    @pyqtSlot()
    def closeCurrentTab(self):
        index = self.ui.tabs.currentIndex()
        if (index == 0):
            # close help browser
            self.hide()
            return
        
        self.closeTab(index)

    @pyqtSlot(int)
    def closeTab(self, index):
        if (index == 0):
            return
        del self.openTabs[index-1]
        self.ui.tabs.removeTab(index)

    @pyqtSlot()
    def openHelpTopicBySender(self):
        sender = self.sender()
        path = str(sender.property('helppath').toString())
        if (not path):
            logger.warning("Bad help topic path: %r", path)
            return

        self.openHelpTopic(path)
        

    @pyqtSlot(QString)
    def openHelpTopic(self, spath):
        path = str(spath)
        pathitems = [x for x in path.split('/') if x];


        # check to see if the tab is already open
        for tab in self.openTabs:
            if (str(tab.property('helppath').toString()) == "/".join(pathitems)):
                # just raise this tab.
                self.ui.tabs.setCurrentWidget(tab)
                return

        widget = self.makeHelpTopicTab(pathitems)
        if (widget is None):
            return
        widget.setProperty('helppath', "/".join(pathitems))

        tabindex = self.ui.tabs.addTab(widget, widget.property('HelpTabTitle').toString())
        self.ui.tabs.setTabToolTip(tabindex, widget.property('HelpTabToolTip').toString())
        self.ui.tabs.setCurrentIndex(tabindex)

        self.openTabs.append(widget)



    def makeHelpTopicTab(self, pathitems):
        if (not len(pathitems)):
            logger.warning("makeHelpTopicTab(): No Path specified!")
            return

        font = settingswidget.get_typewriter_font(self)

        if (pathitems[0] == 'filters'):
            if (len(pathitems) < 2):
                logger.warning("makeHelpTopicTab(): No filter specified!!")
                return
            filtname = pathitems[1]

            tb = QTextBrowser(self.ui.tabs)
            tb.setFont(font)
            tb.setText(filters_factory.format_filter_help(filtname))

            tb.setProperty('HelpTabTitle', '%s filter' %(filtname))
            tb.setProperty('HelpTabToolTip', filters_factory.get_filter_class(filtname).getHelpDescription())
            return tb

        if (pathitems[0] == 'general'):
            if (len(pathitems) < 2):
                logger.warning("makeHelpTopicTab(): No help topic general page specified!!")
                return

            tb = QTextBrowser(self.ui.tabs)
            tb.setFont(font)

            if pathitems[1] == 'welcome':
                tb.setPlainText(HELP_WELCOME)
                tb.setProperty('HelpTabTitle', 'Welcome')
            elif pathitems[1] == 'version':
                tb.setPlainText(argparseactions.helptext_prolog())
                tb.setProperty('HelpTabTitle', 'Version')
            elif pathitems[1] == 'cmdline':
                tb.setPlainText(argparseactions.helptext_prolog() +
                                bibolamazimain.get_args_parser().format_help())
                tb.setProperty('HelpTabTitle', 'Command-Line Help')
            elif pathitems[1] == 'filter-list':
                tb.setPlainText(argparseactions.help_list_filters())
                tb.setProperty('HelpTabTitle', 'Filter List')
            else:
                tb.setPlainText('<Unknown help page>')
                tb.setProperty('HelpTabTitle', '<Unknown>')

            tb.setProperty('HelpTabToolTip', '')
            return tb
                
        logger.warning("makeHelpTopicTab(): Unknown help topic: %r", "/".join(pathitems))
        return None






HELP_WELCOME = ur"""

======================================================================
Bibolamazi -- Prepare consistent BibTeX files for your LaTeX documents
======================================================================

Bibolamazi lets you prepare consistent and uniform BibTeX files for your LaTeX
documents. It lets you prepare your BibTeX entries as you would like them to
be---adding missing or dropping irrelevant information, capitalizing names or
turning them into initials, converting unicode characters to latex escapes, etc.


What Bibolamazi Does
--------------------

Bibolamazi works by reading your reference bibtex files---the ``sources'', which
might for example have been generated by your favorite bibliography manager or
provided by your collaborators---and merging them all into a new file, applying
specific rules, or ``filters'', such as turning all the first names into
initials or normalizing the way arxiv IDs are presented.

The Bibolamazi file is this new file, in which all the required bibtex entries
will be merged. When you prepare you LaTeX document, you should create a new
bibolamazi file, and provide that bibolamazi file as the bibtex file for the
bibliography.

When you open a bibolamazi file, you will be prompted to edit its configuration.
This is the set of rules which will tell bibolamazi where to look for your
bibtex entries and how to handle them. You first need to specify all your
sources, and then all the filters.

The bibolamazi file is then a valid BibTeX file to include into your LaTeX
document, so if your bibolamazi file is named `main.bibolamazi.bib', you would
include the bibliography in your document with a LaTeX command similar to:

    \bibliography{main.bibolamazi}


The Bibolamazi Configuration Section
------------------------------------

If you open the Bibolamazi application and open your bibolamazi file (or create
a new one), you’ll immediately be prompted to edit its configuration section.

Sources are the normal bibtex files from which bibtex entries are read. A source
is specified using the bibolamazi command

  src: source-file.bib  [ alternative-source-file.bib  ... ]

Alternative source locations can be specified, in case the first file does not
exist. This is convenient to locate a file which might be in different locations
on different computers. Each source file name can be an absolute path or a
relative path (relative to the bibolamazi file). It can also be an HTTP URL
which will be downloaded automatically.

You can specify several sources by repeating the src: command.

  src: first-source.bib  alternative-first-source.bib
  src: second-source.bib
  ...

Remember: the *first* readable source of *each* source command will be read, and
merged into the bibolamazi file.

Filters are rules to apply on the whole bibliography database. Their syntax is

  filter: filter_name  <filter-options>

The filter is usually meant to deal with a particular task, such as for example
changing all first names of authors into initials.

For a list of filters and what they do, please refer the first page of this help
browser.

You can usually fine-tune the behavior of the filter by providing options. For
a list of options for a particular filter, please refer again to the help page
of that filter.


What now?
---------

We suggest at this point that you create a new bibolamazi file, and get started
with the serious stuff :)

You might want to have a look at the documentation. It is available at:

  http://bibolamazi.readthedocs.org/en/latest/

If you want an example, you can have a look at the directory

  https://github.com/phfaist/bibolamazi/tree/master/test

and, in particular the bibolamazi files `testX.bibolamazi.bib`.


Command-line
------------

Please note that you can also use bibolamazi in command-line. If you installed
the precompiled application, you'll need to install the command-line version
again. Go to

  https://github.com/phfaist/bibolamazi

and follow the instructions there.

"""



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'sourcelisteditor.ui'
#
# Created: Sun May 31 15:09:06 2015
#      by: PyQt4 UI code generator 4.10.3
#
# WARNING! All changes made in this file will be lost!

from PyQt4 import QtCore, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    def _fromUtf8(s):
        return s

try:
    _encoding = QtGui.QApplication.UnicodeUTF8
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtGui.QApplication.translate(context, text, disambig)

class Ui_SourceListEditor(object):
    def setupUi(self, SourceListEditor):
        SourceListEditor.setObjectName(_fromUtf8("SourceListEditor"))
        SourceListEditor.resize(377, 362)
        self.gridLayout = QtGui.QGridLayout(SourceListEditor)
        self.gridLayout.setObjectName(_fromUtf8("gridLayout"))
        self.lstSources = QtGui.QListWidget(SourceListEditor)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Preferred, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lstSources.sizePolicy().hasHeightForWidth())
        self.lstSources.setSizePolicy(sizePolicy)
        self.lstSources.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.lstSources.setDragDropMode(QtGui.QAbstractItemView.InternalMove)
        self.lstSources.setObjectName(_fromUtf8("lstSources"))
        self.gridLayout.addWidget(self.lstSources, 2, 0, 1, 2)
        self.gbxEditSource = QtGui.QGroupBox(SourceListEditor)
        self.gbxEditSource.setObjectName(_fromUtf8("gbxEditSource"))
        self.gridLayout_2 = QtGui.QGridLayout(self.gbxEditSource)
        self.gridLayout_2.setVerticalSpacing(12)
        self.gridLayout_2.setObjectName(_fromUtf8("gridLayout_2"))
        self.lblFile = QtGui.QLabel(self.gbxEditSource)
        self.lblFile.setObjectName(_fromUtf8("lblFile"))
        self.gridLayout_2.addWidget(self.lblFile, 0, 0, 1, 1)
        self.txtFile = QtGui.QLineEdit(self.gbxEditSource)
        self.txtFile.setObjectName(_fromUtf8("txtFile"))
        self.gridLayout_2.addWidget(self.txtFile, 1, 0, 1, 2)
        self.btnBrowse = QtGui.QPushButton(self.gbxEditSource)
        self.btnBrowse.setObjectName(_fromUtf8("btnBrowse"))
        self.gridLayout_2.addWidget(self.btnBrowse, 0, 1, 1, 1)
        self.gridLayout.addWidget(self.gbxEditSource, 6, 0, 1, 2)
        self.btnAddSource = QtGui.QPushButton(SourceListEditor)
        self.btnAddSource.setObjectName(_fromUtf8("btnAddSource"))
        self.gridLayout.addWidget(self.btnAddSource, 3, 0, 1, 1)
        self.btnRemoveSource = QtGui.QPushButton(SourceListEditor)
        self.btnRemoveSource.setObjectName(_fromUtf8("btnRemoveSource"))
        self.gridLayout.addWidget(self.btnRemoveSource, 3, 1, 1, 1)
        spacerItem = QtGui.QSpacerItem(10, 5, QtGui.QSizePolicy.Minimum, QtGui.QSizePolicy.Fixed)
        self.gridLayout.addItem(spacerItem, 5, 0, 1, 1)
        self.label_2 = QtGui.QLabel(SourceListEditor)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.gridLayout.addWidget(self.label_2, 4, 0, 1, 2)
        self.horizontalLayout = QtGui.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.label = QtGui.QLabel(SourceListEditor)
        self.label.setObjectName(_fromUtf8("label"))
        self.horizontalLayout.addWidget(self.label)
        self.btnAddFavorite = QtGui.QToolButton(SourceListEditor)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8(":/pic/bookmark.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.btnAddFavorite.setIcon(icon)
        self.btnAddFavorite.setObjectName(_fromUtf8("btnAddFavorite"))
        self.horizontalLayout.addWidget(self.btnAddFavorite)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 0, 1, 2)

        self.retranslateUi(SourceListEditor)
        QtCore.QMetaObject.connectSlotsByName(SourceListEditor)

    def retranslateUi(self, SourceListEditor):
        SourceListEditor.setWindowTitle(_translate("SourceListEditor", "Form", None))
        self.lstSources.setToolTip(_translate("SourceListEditor", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Lucida Grande\'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">List of sources to collect bibliographic entries. Reorder entries by <span style=\" font-weight:600;\">Drag&amp;Drop</span>. Remember that the <em>first</em> existing file of <em>each</em> source list will be read.</p></body></html>", None))
        self.gbxEditSource.setTitle(_translate("SourceListEditor", "Edit this source", None))
        self.lblFile.setText(_translate("SourceListEditor", "File or URL:", None))
        self.btnBrowse.setText(_translate("SourceListEditor", "browse file ...", None))
        self.btnAddSource.setText(_translate("SourceListEditor", "Add Source", None))
        self.btnRemoveSource.setText(_translate("SourceListEditor", "Remove this source", None))
        self.label_2.setText(_translate("SourceListEditor", "(reorder sources by drag and drop)", None))
        self.label.setText(_translate("SourceListEditor", "Source List: (first accessible will be used)", None))
        self.btnAddFavorite.setToolTip(_translate("SourceListEditor", "Add this source list to your favorites", None))

from . import bibolamazi_res_rc
